# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: operations.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import errors_pb2 as errors__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from . import lint_pb2 as lint__pb2
from . import shared_pb2 as shared__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x10operations.proto\x12\x06gitaly\x1a\x0c\x65rrors.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\nlint.proto\x1a\x0cshared.proto\"\x8d\x01\n\x17UserCreateBranchRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x13\n\x0b\x62ranch_name\x18\x02 \x01(\x0c\x12\x1a\n\x04user\x18\x03 \x01(\x0b\x32\x0c.gitaly.User\x12\x13\n\x0bstart_point\x18\x04 \x01(\x0c\"S\n\x18UserCreateBranchResponse\x12\x1e\n\x06\x62ranch\x18\x01 \x01(\x0b\x32\x0e.gitaly.BranchJ\x04\x08\x02\x10\x03R\x11pre_receive_error\"P\n\x15UserCreateBranchError\x12.\n\x0b\x63ustom_hook\x18\x01 \x01(\x0b\x32\x17.gitaly.CustomHookErrorH\x00\x42\x07\n\x05\x65rror\"\x98\x01\n\x17UserUpdateBranchRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x13\n\x0b\x62ranch_name\x18\x02 \x01(\x0c\x12\x1a\n\x04user\x18\x03 \x01(\x0b\x32\x0c.gitaly.User\x12\x0e\n\x06newrev\x18\x04 \x01(\x0c\x12\x0e\n\x06oldrev\x18\x05 \x01(\x0c\"5\n\x18UserUpdateBranchResponse\x12\x19\n\x11pre_receive_error\x18\x01 \x01(\t\"\x92\x01\n\x17UserDeleteBranchRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x13\n\x0b\x62ranch_name\x18\x02 \x01(\x0c\x12\x1a\n\x04user\x18\x03 \x01(\x0b\x32\x0c.gitaly.User\x12\x18\n\x10\x65xpected_old_oid\x18\x04 \x01(\t\"3\n\x18UserDeleteBranchResponseJ\x04\x08\x01\x10\x02R\x11pre_receive_error\"\xbc\x01\n\x15UserDeleteBranchError\x12\x30\n\x0c\x61\x63\x63\x65ss_check\x18\x01 \x01(\x0b\x32\x18.gitaly.AccessCheckErrorH\x00\x12\x38\n\x10reference_update\x18\x02 \x01(\x0b\x32\x1c.gitaly.ReferenceUpdateErrorH\x00\x12.\n\x0b\x63ustom_hook\x18\x03 \x01(\x0b\x32\x17.gitaly.CustomHookErrorH\x00\x42\x07\n\x05\x65rror\"\x8c\x01\n\x14UserDeleteTagRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x10\n\x08tag_name\x18\x02 \x01(\x0c\x12\x1a\n\x04user\x18\x03 \x01(\x0b\x32\x0c.gitaly.User\x12\x18\n\x10\x65xpected_old_oid\x18\x04 \x01(\t\"2\n\x15UserDeleteTagResponse\x12\x19\n\x11pre_receive_error\x18\x01 \x01(\t\"\xcb\x01\n\x14UserCreateTagRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x10\n\x08tag_name\x18\x02 \x01(\x0c\x12\x1a\n\x04user\x18\x03 \x01(\x0b\x32\x0c.gitaly.User\x12\x17\n\x0ftarget_revision\x18\x04 \x01(\x0c\x12\x0f\n\x07message\x18\x05 \x01(\x0c\x12-\n\ttimestamp\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"X\n\x15UserCreateTagResponse\x12\x18\n\x03tag\x18\x01 \x01(\x0b\x32\x0b.gitaly.TagJ\x04\x08\x02\x10\x03J\x04\x08\x03\x10\x04R\x06\x65xistsR\x11pre_receive_error\"\xf3\x01\n\x12UserCreateTagError\x12\x30\n\x0c\x61\x63\x63\x65ss_check\x18\x01 \x01(\x0b\x32\x18.gitaly.AccessCheckErrorH\x00\x12\x38\n\x10reference_update\x18\x02 \x01(\x0b\x32\x1c.gitaly.ReferenceUpdateErrorH\x00\x12.\n\x0b\x63ustom_hook\x18\x03 \x01(\x0b\x32\x17.gitaly.CustomHookErrorH\x00\x12\x38\n\x10reference_exists\x18\x04 \x01(\x0b\x32\x1c.gitaly.ReferenceExistsErrorH\x00\x42\x07\n\x05\x65rror\"\xee\x01\n\x16UserMergeBranchRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x1a\n\x04user\x18\x02 \x01(\x0b\x32\x0c.gitaly.User\x12\x11\n\tcommit_id\x18\x03 \x01(\t\x12\x0e\n\x06\x62ranch\x18\x04 \x01(\x0c\x12\x0f\n\x07message\x18\x05 \x01(\x0c\x12-\n\ttimestamp\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x18\n\x10\x65xpected_old_oid\x18\x08 \x01(\t\x12\r\n\x05\x61pply\x18\x06 \x01(\x08\"\x81\x01\n\x17UserMergeBranchResponse\x12\x11\n\tcommit_id\x18\x01 \x01(\t\x12\x34\n\rbranch_update\x18\x03 \x01(\x0b\x32\x1d.gitaly.OperationBranchUpdateJ\x04\x08\x02\x10\x03J\x04\x08\x04\x10\x05R\x11pre_receive_error\"\xf1\x01\n\x14UserMergeBranchError\x12\x30\n\x0c\x61\x63\x63\x65ss_check\x18\x01 \x01(\x0b\x32\x18.gitaly.AccessCheckErrorH\x00\x12\x38\n\x10reference_update\x18\x02 \x01(\x0b\x32\x1c.gitaly.ReferenceUpdateErrorH\x00\x12.\n\x0b\x63ustom_hook\x18\x03 \x01(\x0b\x32\x17.gitaly.CustomHookErrorH\x00\x12\x34\n\x0emerge_conflict\x18\x04 \x01(\x0b\x32\x1a.gitaly.MergeConflictErrorH\x00\x42\x07\n\x05\x65rror\"\xae\x02\n\x15UserMergeToRefRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x1a\n\x04user\x18\x02 \x01(\x0b\x32\x0c.gitaly.User\x12\x12\n\nsource_sha\x18\x03 \x01(\t\x12\x12\n\x06\x62ranch\x18\x04 \x01(\x0c\x42\x02\x18\x01\x12\x12\n\ntarget_ref\x18\x05 \x01(\x0c\x12\x0f\n\x07message\x18\x06 \x01(\x0c\x12\x18\n\x10\x66irst_parent_ref\x18\x07 \x01(\x0c\x12\x1b\n\x0f\x61llow_conflicts\x18\x08 \x01(\x08\x42\x02\x18\x01\x12-\n\ttimestamp\x18\t \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x18\n\x10\x65xpected_old_oid\x18\n \x01(\t\"D\n\x16UserMergeToRefResponse\x12\x11\n\tcommit_id\x18\x01 \x01(\tJ\x04\x08\x02\x10\x03R\x11pre_receive_error\"\xed\x01\n\x16UserRebaseToRefRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x1a\n\x04user\x18\x02 \x01(\x0b\x32\x0c.gitaly.User\x12\x12\n\nsource_sha\x18\x03 \x01(\t\x12\x12\n\ntarget_ref\x18\x05 \x01(\x0c\x12\x18\n\x10\x66irst_parent_ref\x18\x07 \x01(\x0c\x12-\n\ttimestamp\x18\t \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x18\n\x10\x65xpected_old_oid\x18\n \x01(\t\",\n\x17UserRebaseToRefResponse\x12\x11\n\tcommit_id\x18\x01 \x01(\t\"X\n\x15OperationBranchUpdate\x12\x11\n\tcommit_id\x18\x01 \x01(\t\x12\x14\n\x0crepo_created\x18\x02 \x01(\x08\x12\x16\n\x0e\x62ranch_created\x18\x03 \x01(\x08\"\x9c\x01\n\x13UserFFBranchRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x1a\n\x04user\x18\x02 \x01(\x0b\x32\x0c.gitaly.User\x12\x11\n\tcommit_id\x18\x03 \x01(\t\x12\x0e\n\x06\x62ranch\x18\x04 \x01(\x0c\x12\x18\n\x10\x65xpected_old_oid\x18\x05 \x01(\t\"g\n\x14UserFFBranchResponse\x12\x34\n\rbranch_update\x18\x01 \x01(\x0b\x32\x1d.gitaly.OperationBranchUpdate\x12\x19\n\x11pre_receive_error\x18\x02 \x01(\t\"\x86\x03\n\x15UserCherryPickRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x1a\n\x04user\x18\x02 \x01(\x0b\x32\x0c.gitaly.User\x12!\n\x06\x63ommit\x18\x03 \x01(\x0b\x32\x11.gitaly.GitCommit\x12\x13\n\x0b\x62ranch_name\x18\x04 \x01(\x0c\x12\x0f\n\x07message\x18\x05 \x01(\x0c\x12\x19\n\x11start_branch_name\x18\x06 \x01(\x0c\x12,\n\x10start_repository\x18\x07 \x01(\x0b\x32\x12.gitaly.Repository\x12\x0f\n\x07\x64ry_run\x18\x08 \x01(\x08\x12-\n\ttimestamp\x18\t \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x18\n\x10\x65xpected_old_oid\x18\n \x01(\t\x12\x1a\n\x12\x63ommit_author_name\x18\x0b \x01(\x0c\x12\x1b\n\x13\x63ommit_author_email\x18\x0c \x01(\x0c\"\xb2\x01\n\x16UserCherryPickResponse\x12\x34\n\rbranch_update\x18\x01 \x01(\x0b\x32\x1d.gitaly.OperationBranchUpdateJ\x04\x08\x02\x10\x03J\x04\x08\x03\x10\x04J\x04\x08\x04\x10\x05J\x04\x08\x05\x10\x06R\x11\x63reate_tree_errorR\x0c\x63ommit_errorR\x11pre_receive_errorR\x16\x63reate_tree_error_code\"\x8f\x02\n\x13UserCherryPickError\x12:\n\x14\x63herry_pick_conflict\x18\x01 \x01(\x0b\x32\x1a.gitaly.MergeConflictErrorH\x00\x12:\n\x16target_branch_diverged\x18\x02 \x01(\x0b\x32\x18.gitaly.NotAncestorErrorH\x00\x12\x45\n\x17\x63hanges_already_applied\x18\x03 \x01(\x0b\x32\".gitaly.ChangesAlreadyAppliedErrorH\x00\x12\x30\n\x0c\x61\x63\x63\x65ss_check\x18\x04 \x01(\x0b\x32\x18.gitaly.AccessCheckErrorH\x00\x42\x07\n\x05\x65rror\"\xc9\x02\n\x11UserRevertRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x1a\n\x04user\x18\x02 \x01(\x0b\x32\x0c.gitaly.User\x12!\n\x06\x63ommit\x18\x03 \x01(\x0b\x32\x11.gitaly.GitCommit\x12\x13\n\x0b\x62ranch_name\x18\x04 \x01(\x0c\x12\x0f\n\x07message\x18\x05 \x01(\x0c\x12\x19\n\x11start_branch_name\x18\x06 \x01(\x0c\x12,\n\x10start_repository\x18\x07 \x01(\x0b\x32\x12.gitaly.Repository\x12\x0f\n\x07\x64ry_run\x18\x08 \x01(\x08\x12-\n\ttimestamp\x18\t \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x18\n\x10\x65xpected_old_oid\x18\n \x01(\t\"\x98\x02\n\x12UserRevertResponse\x12\x34\n\rbranch_update\x18\x01 \x01(\x0b\x32\x1d.gitaly.OperationBranchUpdate\x12\x19\n\x11\x63reate_tree_error\x18\x02 \x01(\t\x12\x14\n\x0c\x63ommit_error\x18\x03 \x01(\t\x12\x19\n\x11pre_receive_error\x18\x04 \x01(\t\x12J\n\x16\x63reate_tree_error_code\x18\x05 \x01(\x0e\x32*.gitaly.UserRevertResponse.CreateTreeError\"4\n\x0f\x43reateTreeError\x12\x08\n\x04NONE\x10\x00\x12\t\n\x05\x45MPTY\x10\x01\x12\x0c\n\x08\x43ONFLICT\x10\x02\"\xf9\x01\n\x0fUserRevertError\x12\x34\n\x0emerge_conflict\x18\x01 \x01(\x0b\x32\x1a.gitaly.MergeConflictErrorH\x00\x12\x45\n\x17\x63hanges_already_applied\x18\x02 \x01(\x0b\x32\".gitaly.ChangesAlreadyAppliedErrorH\x00\x12.\n\x0b\x63ustom_hook\x18\x03 \x01(\x0b\x32\x17.gitaly.CustomHookErrorH\x00\x12\x30\n\x0cnot_ancestor\x18\x04 \x01(\x0b\x32\x18.gitaly.NotAncestorErrorH\x00\x42\x07\n\x05\x65rror\"\xa7\x02\n\x1bUserCommitFilesActionHeader\x12>\n\x06\x61\x63tion\x18\x01 \x01(\x0e\x32..gitaly.UserCommitFilesActionHeader.ActionType\x12\x11\n\tfile_path\x18\x02 \x01(\x0c\x12\x15\n\rprevious_path\x18\x03 \x01(\x0c\x12\x16\n\x0e\x62\x61se64_content\x18\x04 \x01(\x08\x12\x18\n\x10\x65xecute_filemode\x18\x05 \x01(\x08\x12\x15\n\rinfer_content\x18\x06 \x01(\x08\"U\n\nActionType\x12\n\n\x06\x43REATE\x10\x00\x12\x0e\n\nCREATE_DIR\x10\x01\x12\n\n\x06UPDATE\x10\x02\x12\x08\n\x04MOVE\x10\x03\x12\n\n\x06\x44\x45LETE\x10\x04\x12\t\n\x05\x43HMOD\x10\x05\"\x85\x01\n\x15UserCommitFilesAction\x12\x35\n\x06header\x18\x01 \x01(\x0b\x32#.gitaly.UserCommitFilesActionHeaderH\x00\x12\x11\n\x07\x63ontent\x18\x02 \x01(\x0cH\x00\x42\"\n user_commit_files_action_payload\"\x90\x03\n\x1cUserCommitFilesRequestHeader\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x1a\n\x04user\x18\x02 \x01(\x0b\x32\x0c.gitaly.User\x12\x13\n\x0b\x62ranch_name\x18\x03 \x01(\x0c\x12\x16\n\x0e\x63ommit_message\x18\x04 \x01(\x0c\x12\x1a\n\x12\x63ommit_author_name\x18\x05 \x01(\x0c\x12\x1b\n\x13\x63ommit_author_email\x18\x06 \x01(\x0c\x12\x19\n\x11start_branch_name\x18\x07 \x01(\x0c\x12,\n\x10start_repository\x18\x08 \x01(\x0b\x32\x12.gitaly.Repository\x12\r\n\x05\x66orce\x18\t \x01(\x08\x12\x11\n\tstart_sha\x18\n \x01(\t\x12-\n\ttimestamp\x18\x0b \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x18\n\x10\x65xpected_old_oid\x18\x0c \x01(\t\x12\x0c\n\x04sign\x18\r \x01(\x08\"\xa6\x01\n\x16UserCommitFilesRequest\x12\x36\n\x06header\x18\x01 \x01(\x0b\x32$.gitaly.UserCommitFilesRequestHeaderH\x00\x12/\n\x06\x61\x63tion\x18\x02 \x01(\x0b\x32\x1d.gitaly.UserCommitFilesActionH\x00\x42#\n!user_commit_files_request_payload\"\x7f\n\x17UserCommitFilesResponse\x12\x34\n\rbranch_update\x18\x01 \x01(\x0b\x32\x1d.gitaly.OperationBranchUpdate\x12\x13\n\x0bindex_error\x18\x02 \x01(\t\x12\x19\n\x11pre_receive_error\x18\x03 \x01(\t\"\xad\x01\n\x14UserCommitFilesError\x12\x30\n\x0c\x61\x63\x63\x65ss_check\x18\x01 \x01(\x0b\x32\x18.gitaly.AccessCheckErrorH\x00\x12*\n\x0cindex_update\x18\x02 \x01(\x0b\x32\x12.gitaly.IndexErrorH\x00\x12.\n\x0b\x63ustom_hook\x18\x03 \x01(\x0b\x32\x17.gitaly.CustomHookErrorH\x00\x42\x07\n\x05\x65rror\"\xb8\x03\n\x1cUserRebaseConfirmableRequest\x12=\n\x06header\x18\x01 \x01(\x0b\x32+.gitaly.UserRebaseConfirmableRequest.HeaderH\x00\x12\x0f\n\x05\x61pply\x18\x02 \x01(\x08H\x00\x1a\x9c\x02\n\x06Header\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x1a\n\x04user\x18\x02 \x01(\x0b\x32\x0c.gitaly.User\x12\x15\n\trebase_id\x18\x03 \x01(\tB\x02\x18\x01\x12\x0e\n\x06\x62ranch\x18\x04 \x01(\x0c\x12\x12\n\nbranch_sha\x18\x05 \x01(\t\x12-\n\x11remote_repository\x18\x06 \x01(\x0b\x32\x12.gitaly.Repository\x12\x15\n\rremote_branch\x18\x07 \x01(\x0c\x12\x18\n\x10git_push_options\x18\x08 \x03(\t\x12-\n\ttimestamp\x18\t \x01(\x0b\x32\x1a.google.protobuf.TimestampB)\n\'user_rebase_confirmable_request_payload\"\xa5\x01\n\x1dUserRebaseConfirmableResponse\x12\x14\n\nrebase_sha\x18\x01 \x01(\tH\x00\x12\x18\n\x0erebase_applied\x18\x02 \x01(\x08H\x00\x42*\n(user_rebase_confirmable_response_payloadJ\x04\x08\x03\x10\x04J\x04\x08\x04\x10\x05R\x11pre_receive_errorR\tgit_error\"\xfd\x01\n\x11UserSquashRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x1a\n\x04user\x18\x02 \x01(\x0b\x32\x0c.gitaly.User\x12\x11\n\tstart_sha\x18\x05 \x01(\t\x12\x0f\n\x07\x65nd_sha\x18\x06 \x01(\t\x12\x1c\n\x06\x61uthor\x18\x07 \x01(\x0b\x32\x0c.gitaly.User\x12\x16\n\x0e\x63ommit_message\x18\x08 \x01(\x0c\x12-\n\ttimestamp\x18\t \x01(\x0b\x32\x1a.google.protobuf.TimestampJ\x04\x08\x03\x10\x04J\x04\x08\x04\x10\x05R\tsquash_id\"R\n\x12UserSquashResponse\x12\x12\n\nsquash_sha\x18\x01 \x01(\tJ\x04\x08\x02\x10\x03J\x04\x08\x03\x10\x04R\x11pre_receive_errorR\tgit_error\"\x8e\x01\n\x1aUserRebaseConfirmableError\x12\x35\n\x0frebase_conflict\x18\x01 \x01(\x0b\x32\x1a.gitaly.MergeConflictErrorH\x00\x12\x30\n\x0c\x61\x63\x63\x65ss_check\x18\x02 \x01(\x0b\x32\x18.gitaly.AccessCheckErrorH\x00\x42\x07\n\x05\x65rror\"\x8b\x01\n\x0fUserSquashError\x12\x38\n\x10resolve_revision\x18\x01 \x01(\x0b\x32\x1c.gitaly.ResolveRevisionErrorH\x00\x12\x35\n\x0frebase_conflict\x18\x02 \x01(\x0b\x32\x1a.gitaly.MergeConflictErrorH\x00\x42\x07\n\x05\x65rror\"\xbb\x02\n\x15UserApplyPatchRequest\x12\x36\n\x06header\x18\x01 \x01(\x0b\x32$.gitaly.UserApplyPatchRequest.HeaderH\x00\x12\x11\n\x07patches\x18\x02 \x01(\x0cH\x00\x1a\xb2\x01\n\x06Header\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x1a\n\x04user\x18\x02 \x01(\x0b\x32\x0c.gitaly.User\x12\x15\n\rtarget_branch\x18\x03 \x01(\x0c\x12-\n\ttimestamp\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x18\n\x10\x65xpected_old_oid\x18\x05 \x01(\tB\"\n user_apply_patch_request_payload\"N\n\x16UserApplyPatchResponse\x12\x34\n\rbranch_update\x18\x01 \x01(\x0b\x32\x1d.gitaly.OperationBranchUpdate\"\xfe\x01\n\x1aUserUpdateSubmoduleRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x1a\n\x04user\x18\x02 \x01(\x0b\x32\x0c.gitaly.User\x12\x12\n\ncommit_sha\x18\x03 \x01(\t\x12\x0e\n\x06\x62ranch\x18\x04 \x01(\x0c\x12\x11\n\tsubmodule\x18\x05 \x01(\x0c\x12\x16\n\x0e\x63ommit_message\x18\x06 \x01(\x0c\x12-\n\ttimestamp\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x18\n\x10\x65xpected_old_oid\x18\x08 \x01(\t\"\x9d\x01\n\x1bUserUpdateSubmoduleResponse\x12\x34\n\rbranch_update\x18\x01 \x01(\x0b\x32\x1d.gitaly.OperationBranchUpdate\x12\x19\n\x11pre_receive_error\x18\x02 \x01(\t\x12\x14\n\x0c\x63ommit_error\x18\x04 \x01(\tJ\x04\x08\x03\x10\x04R\x11\x63reate_tree_error2\xc9\x0b\n\x10OperationService\x12]\n\x10UserCreateBranch\x12\x1f.gitaly.UserCreateBranchRequest\x1a .gitaly.UserCreateBranchResponse\"\x06\xfa\x97(\x02\x08\x01\x12]\n\x10UserUpdateBranch\x12\x1f.gitaly.UserUpdateBranchRequest\x1a .gitaly.UserUpdateBranchResponse\"\x06\xfa\x97(\x02\x08\x01\x12]\n\x10UserDeleteBranch\x12\x1f.gitaly.UserDeleteBranchRequest\x1a .gitaly.UserDeleteBranchResponse\"\x06\xfa\x97(\x02\x08\x01\x12T\n\rUserCreateTag\x12\x1c.gitaly.UserCreateTagRequest\x1a\x1d.gitaly.UserCreateTagResponse\"\x06\xfa\x97(\x02\x08\x01\x12T\n\rUserDeleteTag\x12\x1c.gitaly.UserDeleteTagRequest\x1a\x1d.gitaly.UserDeleteTagResponse\"\x06\xfa\x97(\x02\x08\x01\x12W\n\x0eUserMergeToRef\x12\x1d.gitaly.UserMergeToRefRequest\x1a\x1e.gitaly.UserMergeToRefResponse\"\x06\xfa\x97(\x02\x08\x01\x12Z\n\x0fUserRebaseToRef\x12\x1e.gitaly.UserRebaseToRefRequest\x1a\x1f.gitaly.UserRebaseToRefResponse\"\x06\xfa\x97(\x02\x08\x01\x12^\n\x0fUserMergeBranch\x12\x1e.gitaly.UserMergeBranchRequest\x1a\x1f.gitaly.UserMergeBranchResponse\"\x06\xfa\x97(\x02\x08\x01(\x01\x30\x01\x12Q\n\x0cUserFFBranch\x12\x1b.gitaly.UserFFBranchRequest\x1a\x1c.gitaly.UserFFBranchResponse\"\x06\xfa\x97(\x02\x08\x01\x12W\n\x0eUserCherryPick\x12\x1d.gitaly.UserCherryPickRequest\x1a\x1e.gitaly.UserCherryPickResponse\"\x06\xfa\x97(\x02\x08\x01\x12\\\n\x0fUserCommitFiles\x12\x1e.gitaly.UserCommitFilesRequest\x1a\x1f.gitaly.UserCommitFilesResponse\"\x06\xfa\x97(\x02\x08\x01(\x01\x12p\n\x15UserRebaseConfirmable\x12$.gitaly.UserRebaseConfirmableRequest\x1a%.gitaly.UserRebaseConfirmableResponse\"\x06\xfa\x97(\x02\x08\x01(\x01\x30\x01\x12K\n\nUserRevert\x12\x19.gitaly.UserRevertRequest\x1a\x1a.gitaly.UserRevertResponse\"\x06\xfa\x97(\x02\x08\x01\x12K\n\nUserSquash\x12\x19.gitaly.UserSquashRequest\x1a\x1a.gitaly.UserSquashResponse\"\x06\xfa\x97(\x02\x08\x01\x12Y\n\x0eUserApplyPatch\x12\x1d.gitaly.UserApplyPatchRequest\x1a\x1e.gitaly.UserApplyPatchResponse\"\x06\xfa\x97(\x02\x08\x01(\x01\x12\x66\n\x13UserUpdateSubmodule\x12\".gitaly.UserUpdateSubmoduleRequest\x1a#.gitaly.UserUpdateSubmoduleResponse\"\x06\xfa\x97(\x02\x08\x01\x42\x34Z2gitlab.com/gitlab-org/gitaly/v16/proto/go/gitalypbb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'operations_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z2gitlab.com/gitlab-org/gitaly/v16/proto/go/gitalypb'
  _USERCREATEBRANCHREQUEST.fields_by_name['repository']._options = None
  _USERCREATEBRANCHREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _USERUPDATEBRANCHREQUEST.fields_by_name['repository']._options = None
  _USERUPDATEBRANCHREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _USERDELETEBRANCHREQUEST.fields_by_name['repository']._options = None
  _USERDELETEBRANCHREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _USERDELETETAGREQUEST.fields_by_name['repository']._options = None
  _USERDELETETAGREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _USERCREATETAGREQUEST.fields_by_name['repository']._options = None
  _USERCREATETAGREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _USERMERGEBRANCHREQUEST.fields_by_name['repository']._options = None
  _USERMERGEBRANCHREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _USERMERGETOREFREQUEST.fields_by_name['repository']._options = None
  _USERMERGETOREFREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _USERMERGETOREFREQUEST.fields_by_name['branch']._options = None
  _USERMERGETOREFREQUEST.fields_by_name['branch']._serialized_options = b'\030\001'
  _USERMERGETOREFREQUEST.fields_by_name['allow_conflicts']._options = None
  _USERMERGETOREFREQUEST.fields_by_name['allow_conflicts']._serialized_options = b'\030\001'
  _USERREBASETOREFREQUEST.fields_by_name['repository']._options = None
  _USERREBASETOREFREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _USERFFBRANCHREQUEST.fields_by_name['repository']._options = None
  _USERFFBRANCHREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _USERCHERRYPICKREQUEST.fields_by_name['repository']._options = None
  _USERCHERRYPICKREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _USERREVERTREQUEST.fields_by_name['repository']._options = None
  _USERREVERTREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _USERCOMMITFILESREQUESTHEADER.fields_by_name['repository']._options = None
  _USERCOMMITFILESREQUESTHEADER.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _USERREBASECONFIRMABLEREQUEST_HEADER.fields_by_name['repository']._options = None
  _USERREBASECONFIRMABLEREQUEST_HEADER.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _USERREBASECONFIRMABLEREQUEST_HEADER.fields_by_name['rebase_id']._options = None
  _USERREBASECONFIRMABLEREQUEST_HEADER.fields_by_name['rebase_id']._serialized_options = b'\030\001'
  _USERSQUASHREQUEST.fields_by_name['repository']._options = None
  _USERSQUASHREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _USERAPPLYPATCHREQUEST_HEADER.fields_by_name['repository']._options = None
  _USERAPPLYPATCHREQUEST_HEADER.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _USERUPDATESUBMODULEREQUEST.fields_by_name['repository']._options = None
  _USERUPDATESUBMODULEREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _OPERATIONSERVICE.methods_by_name['UserCreateBranch']._options = None
  _OPERATIONSERVICE.methods_by_name['UserCreateBranch']._serialized_options = b'\372\227(\002\010\001'
  _OPERATIONSERVICE.methods_by_name['UserUpdateBranch']._options = None
  _OPERATIONSERVICE.methods_by_name['UserUpdateBranch']._serialized_options = b'\372\227(\002\010\001'
  _OPERATIONSERVICE.methods_by_name['UserDeleteBranch']._options = None
  _OPERATIONSERVICE.methods_by_name['UserDeleteBranch']._serialized_options = b'\372\227(\002\010\001'
  _OPERATIONSERVICE.methods_by_name['UserCreateTag']._options = None
  _OPERATIONSERVICE.methods_by_name['UserCreateTag']._serialized_options = b'\372\227(\002\010\001'
  _OPERATIONSERVICE.methods_by_name['UserDeleteTag']._options = None
  _OPERATIONSERVICE.methods_by_name['UserDeleteTag']._serialized_options = b'\372\227(\002\010\001'
  _OPERATIONSERVICE.methods_by_name['UserMergeToRef']._options = None
  _OPERATIONSERVICE.methods_by_name['UserMergeToRef']._serialized_options = b'\372\227(\002\010\001'
  _OPERATIONSERVICE.methods_by_name['UserRebaseToRef']._options = None
  _OPERATIONSERVICE.methods_by_name['UserRebaseToRef']._serialized_options = b'\372\227(\002\010\001'
  _OPERATIONSERVICE.methods_by_name['UserMergeBranch']._options = None
  _OPERATIONSERVICE.methods_by_name['UserMergeBranch']._serialized_options = b'\372\227(\002\010\001'
  _OPERATIONSERVICE.methods_by_name['UserFFBranch']._options = None
  _OPERATIONSERVICE.methods_by_name['UserFFBranch']._serialized_options = b'\372\227(\002\010\001'
  _OPERATIONSERVICE.methods_by_name['UserCherryPick']._options = None
  _OPERATIONSERVICE.methods_by_name['UserCherryPick']._serialized_options = b'\372\227(\002\010\001'
  _OPERATIONSERVICE.methods_by_name['UserCommitFiles']._options = None
  _OPERATIONSERVICE.methods_by_name['UserCommitFiles']._serialized_options = b'\372\227(\002\010\001'
  _OPERATIONSERVICE.methods_by_name['UserRebaseConfirmable']._options = None
  _OPERATIONSERVICE.methods_by_name['UserRebaseConfirmable']._serialized_options = b'\372\227(\002\010\001'
  _OPERATIONSERVICE.methods_by_name['UserRevert']._options = None
  _OPERATIONSERVICE.methods_by_name['UserRevert']._serialized_options = b'\372\227(\002\010\001'
  _OPERATIONSERVICE.methods_by_name['UserSquash']._options = None
  _OPERATIONSERVICE.methods_by_name['UserSquash']._serialized_options = b'\372\227(\002\010\001'
  _OPERATIONSERVICE.methods_by_name['UserApplyPatch']._options = None
  _OPERATIONSERVICE.methods_by_name['UserApplyPatch']._serialized_options = b'\372\227(\002\010\001'
  _OPERATIONSERVICE.methods_by_name['UserUpdateSubmodule']._options = None
  _OPERATIONSERVICE.methods_by_name['UserUpdateSubmodule']._serialized_options = b'\372\227(\002\010\001'
  _globals['_USERCREATEBRANCHREQUEST']._serialized_start=102
  _globals['_USERCREATEBRANCHREQUEST']._serialized_end=243
  _globals['_USERCREATEBRANCHRESPONSE']._serialized_start=245
  _globals['_USERCREATEBRANCHRESPONSE']._serialized_end=328
  _globals['_USERCREATEBRANCHERROR']._serialized_start=330
  _globals['_USERCREATEBRANCHERROR']._serialized_end=410
  _globals['_USERUPDATEBRANCHREQUEST']._serialized_start=413
  _globals['_USERUPDATEBRANCHREQUEST']._serialized_end=565
  _globals['_USERUPDATEBRANCHRESPONSE']._serialized_start=567
  _globals['_USERUPDATEBRANCHRESPONSE']._serialized_end=620
  _globals['_USERDELETEBRANCHREQUEST']._serialized_start=623
  _globals['_USERDELETEBRANCHREQUEST']._serialized_end=769
  _globals['_USERDELETEBRANCHRESPONSE']._serialized_start=771
  _globals['_USERDELETEBRANCHRESPONSE']._serialized_end=822
  _globals['_USERDELETEBRANCHERROR']._serialized_start=825
  _globals['_USERDELETEBRANCHERROR']._serialized_end=1013
  _globals['_USERDELETETAGREQUEST']._serialized_start=1016
  _globals['_USERDELETETAGREQUEST']._serialized_end=1156
  _globals['_USERDELETETAGRESPONSE']._serialized_start=1158
  _globals['_USERDELETETAGRESPONSE']._serialized_end=1208
  _globals['_USERCREATETAGREQUEST']._serialized_start=1211
  _globals['_USERCREATETAGREQUEST']._serialized_end=1414
  _globals['_USERCREATETAGRESPONSE']._serialized_start=1416
  _globals['_USERCREATETAGRESPONSE']._serialized_end=1504
  _globals['_USERCREATETAGERROR']._serialized_start=1507
  _globals['_USERCREATETAGERROR']._serialized_end=1750
  _globals['_USERMERGEBRANCHREQUEST']._serialized_start=1753
  _globals['_USERMERGEBRANCHREQUEST']._serialized_end=1991
  _globals['_USERMERGEBRANCHRESPONSE']._serialized_start=1994
  _globals['_USERMERGEBRANCHRESPONSE']._serialized_end=2123
  _globals['_USERMERGEBRANCHERROR']._serialized_start=2126
  _globals['_USERMERGEBRANCHERROR']._serialized_end=2367
  _globals['_USERMERGETOREFREQUEST']._serialized_start=2370
  _globals['_USERMERGETOREFREQUEST']._serialized_end=2672
  _globals['_USERMERGETOREFRESPONSE']._serialized_start=2674
  _globals['_USERMERGETOREFRESPONSE']._serialized_end=2742
  _globals['_USERREBASETOREFREQUEST']._serialized_start=2745
  _globals['_USERREBASETOREFREQUEST']._serialized_end=2982
  _globals['_USERREBASETOREFRESPONSE']._serialized_start=2984
  _globals['_USERREBASETOREFRESPONSE']._serialized_end=3028
  _globals['_OPERATIONBRANCHUPDATE']._serialized_start=3030
  _globals['_OPERATIONBRANCHUPDATE']._serialized_end=3118
  _globals['_USERFFBRANCHREQUEST']._serialized_start=3121
  _globals['_USERFFBRANCHREQUEST']._serialized_end=3277
  _globals['_USERFFBRANCHRESPONSE']._serialized_start=3279
  _globals['_USERFFBRANCHRESPONSE']._serialized_end=3382
  _globals['_USERCHERRYPICKREQUEST']._serialized_start=3385
  _globals['_USERCHERRYPICKREQUEST']._serialized_end=3775
  _globals['_USERCHERRYPICKRESPONSE']._serialized_start=3778
  _globals['_USERCHERRYPICKRESPONSE']._serialized_end=3956
  _globals['_USERCHERRYPICKERROR']._serialized_start=3959
  _globals['_USERCHERRYPICKERROR']._serialized_end=4230
  _globals['_USERREVERTREQUEST']._serialized_start=4233
  _globals['_USERREVERTREQUEST']._serialized_end=4562
  _globals['_USERREVERTRESPONSE']._serialized_start=4565
  _globals['_USERREVERTRESPONSE']._serialized_end=4845
  _globals['_USERREVERTRESPONSE_CREATETREEERROR']._serialized_start=4793
  _globals['_USERREVERTRESPONSE_CREATETREEERROR']._serialized_end=4845
  _globals['_USERREVERTERROR']._serialized_start=4848
  _globals['_USERREVERTERROR']._serialized_end=5097
  _globals['_USERCOMMITFILESACTIONHEADER']._serialized_start=5100
  _globals['_USERCOMMITFILESACTIONHEADER']._serialized_end=5395
  _globals['_USERCOMMITFILESACTIONHEADER_ACTIONTYPE']._serialized_start=5310
  _globals['_USERCOMMITFILESACTIONHEADER_ACTIONTYPE']._serialized_end=5395
  _globals['_USERCOMMITFILESACTION']._serialized_start=5398
  _globals['_USERCOMMITFILESACTION']._serialized_end=5531
  _globals['_USERCOMMITFILESREQUESTHEADER']._serialized_start=5534
  _globals['_USERCOMMITFILESREQUESTHEADER']._serialized_end=5934
  _globals['_USERCOMMITFILESREQUEST']._serialized_start=5937
  _globals['_USERCOMMITFILESREQUEST']._serialized_end=6103
  _globals['_USERCOMMITFILESRESPONSE']._serialized_start=6105
  _globals['_USERCOMMITFILESRESPONSE']._serialized_end=6232
  _globals['_USERCOMMITFILESERROR']._serialized_start=6235
  _globals['_USERCOMMITFILESERROR']._serialized_end=6408
  _globals['_USERREBASECONFIRMABLEREQUEST']._serialized_start=6411
  _globals['_USERREBASECONFIRMABLEREQUEST']._serialized_end=6851
  _globals['_USERREBASECONFIRMABLEREQUEST_HEADER']._serialized_start=6524
  _globals['_USERREBASECONFIRMABLEREQUEST_HEADER']._serialized_end=6808
  _globals['_USERREBASECONFIRMABLERESPONSE']._serialized_start=6854
  _globals['_USERREBASECONFIRMABLERESPONSE']._serialized_end=7019
  _globals['_USERSQUASHREQUEST']._serialized_start=7022
  _globals['_USERSQUASHREQUEST']._serialized_end=7275
  _globals['_USERSQUASHRESPONSE']._serialized_start=7277
  _globals['_USERSQUASHRESPONSE']._serialized_end=7359
  _globals['_USERREBASECONFIRMABLEERROR']._serialized_start=7362
  _globals['_USERREBASECONFIRMABLEERROR']._serialized_end=7504
  _globals['_USERSQUASHERROR']._serialized_start=7507
  _globals['_USERSQUASHERROR']._serialized_end=7646
  _globals['_USERAPPLYPATCHREQUEST']._serialized_start=7649
  _globals['_USERAPPLYPATCHREQUEST']._serialized_end=7964
  _globals['_USERAPPLYPATCHREQUEST_HEADER']._serialized_start=7750
  _globals['_USERAPPLYPATCHREQUEST_HEADER']._serialized_end=7928
  _globals['_USERAPPLYPATCHRESPONSE']._serialized_start=7966
  _globals['_USERAPPLYPATCHRESPONSE']._serialized_end=8044
  _globals['_USERUPDATESUBMODULEREQUEST']._serialized_start=8047
  _globals['_USERUPDATESUBMODULEREQUEST']._serialized_end=8301
  _globals['_USERUPDATESUBMODULERESPONSE']._serialized_start=8304
  _globals['_USERUPDATESUBMODULERESPONSE']._serialized_end=8461
  _globals['_OPERATIONSERVICE']._serialized_start=8464
  _globals['_OPERATIONSERVICE']._serialized_end=9945
# @@protoc_insertion_point(module_scope)



# Copyright 2020 Georges Racinet <georges.racinet@octobus.net>
#
# This software may be used and distributed according to the terms of the
# GNU General Public License version 2 or any later version.
#
# SPDX-License-Identifier: GPL-2.0-or-later
from copy import deepcopy
import random

from heptapod.testhelpers import LocalRepoWrapper

from ..stub.shared_pb2 import Repository

MINIMAL_HG_CONFIG = dict(extensions=dict(topic='',
                                         evolve='',
                                         heptapod='',
                                         hgitaly='',
                                         ))

HG_CONFIG_WITH_STATE_MAINTAINER = deepcopy(MINIMAL_HG_CONFIG)
HG_CONFIG_WITH_STATE_MAINTAINER.update(
    hooks={
        'pretxnclose.heptapod_sync':
            'python:heptapod.hooks.gitlab_mirror.mirror'
    }
)


def make_empty_repo(storages_root, relative_path=None, storage='default',
                    config=None,
                    extra_extensions=()):
    """Create an empty repo, ready for most of HGitaly tests.

    :relative_path: path to the repo, relative to server_repos_root, and
                    hence also relative path in the sense of Gitaly protocol.
                    Can be given as :class:`str` or :class:`pathlib.Path`.
                    If ``None``, a path is randomly chosen.
    :storage: name of the target storage, will have to be created and
              registered before hand on the testing grpc server if needed.

    The path to the repository root can be retrieved from the returned
    wrapper as the :class:`pathlib.Path` instance `path` attribute.

    return: wrapper, gRPC Repository message
    """
    if relative_path is None:
        relative_path = 'repo-' + hex(random.getrandbits(64))[2:]
    repo_path = storages_root / storage / relative_path
    if config is None:
        config = MINIMAL_HG_CONFIG

    config['extensions'].update((ext, '') for ext in extra_extensions)
    wrapper = LocalRepoWrapper.init(repo_path, config=config)
    wrapper.write_hgrc(config)

    grpc_repo = Repository(relative_path=str(relative_path),
                           storage_name=storage)
    return wrapper, grpc_repo


def make_tree_shaped_repo(storages_root, **kw):
    """Make a testing repo with a tree shape.

    :return: ``(wrapper, grpc_repo, changesets)``, where ``wrapper`` and
             ``grpc_repo`` are as in :func:`make_empy_repo` and ``changesets``
             is a :class:`dict` whose values are interesting changesets,
             given as :class:`changectx instances`

    Here's the graphlog, with added annotation for keys of the ``changesets``
    :class:`dict`::

      $ hg log -G -T '{desc}\nbranch & topic: {branch}:{topic}\n\n'
      @  Topic head  KEY: top2
      |  branch & topic: default:zzetop
      |
      o  Topic first  KEY: top1
      |  branch & topic: default:zzetop
      |
      | o  Other main wild head  KEY: wild2
      | |  branch & topic: other:
      | |
      | | o  Other wild  KEY: wild1
      | |/   branch & topic: other:
      | |
      | o  Start other  KEY: other_base
      |/   branch & topic: other:
      |
      | o  Head of default  KEY: default
      |/   branch & topic: default:
      |
      o  Base  KEY: base
         branch & topic: default:
    """
    wrapper, grpc_repo = make_empty_repo(storages_root, **kw)
    chgs = {}
    base = chgs['base'] = wrapper.write_commit('foo', message='Base')
    chgs['default'] = wrapper.write_commit('foo', message='Head of default')
    other_base = chgs['other_base'] = wrapper.write_commit(
        'foo', message='Start other', branch='other', parent=base)
    chgs['wild1'] = wrapper.write_commit('foo', message='Other wild',
                                         branch='other')
    chgs['wild2'] = wrapper.write_commit('foo', message='Other main wild head',
                                         branch='other', parent=other_base)
    chgs['top1'] = wrapper.write_commit('foo', message='Topic first',
                                        branch='default', topic='zzetop',
                                        parent=base)
    chgs['top2'] = wrapper.write_commit('foo', message='Topic head')
    return wrapper, grpc_repo, chgs


def make_empty_repo_uniform(*args, **kwargs):
    """A version with same signature as functions that also return changesets.
    """
    return make_empty_repo(*args, **kwargs) + ({}, )


def make_empty_repo_with_gitlab_state_maintainer(*args, **kwargs):
    kwargs['config'] = HG_CONFIG_WITH_STATE_MAINTAINER
    return make_empty_repo_uniform(*args, **kwargs)



# Copyright 2022 Georges Racinet <georges.racinet@octobus.net>
#
# This software may be used and distributed according to the terms of the
# GNU General Public License version 2 or any later version.
#
# SPDX-License-Identifier: GPL-2.0-or-later
"""Mercurial bundle utilities meant for tests.

"""
from mercurial import (
    changegroup,
    exchange,
    hg,
    ui as uimod,
)
from mercurial.node import hex as node_hex
from mercurial_testhelpers import (
    as_bytes,
)


def list_bundle_contents(path):
    """Simple utility to list the contents.

    Inspired from mercurial.debugcommands. As usual Mercurial provides
    not much between very low level binary streaming methods meant for
    actual application or UI methods meant to write to stdout.
    """
    nodes = []
    ui = uimod.ui()
    with hg.openpath(ui, as_bytes(path)) as bundle:
        gen = exchange.readbundle(ui, bundle, as_bytes(path.basename))
        for part in gen.iterparts():
            if part.type == b'changegroup':
                version = part.params.get(b'version', b'01')
                cg = changegroup.getunbundler(version, part, b'UN')
                cg.changelogheader()
                nodes.extend(node_hex(delta[0]) for delta in cg.deltaiter())
    return dict(nodes=nodes)



# Copyright 2020-2022 Georges Racinet <georges.racinet@octobus.net>
#
# This software may be used and distributed according to the terms of the
# GNU General Public License version 2 or any later version.
#
# SPDX-License-Identifier: GPL-2.0-or-later
"""Listen address commmon utilities.
"""
import socket


DEFAULT_TCP_PORT = 9237


class UnsupportedUrlScheme(ValueError):
    pass


class InvalidUrl(ValueError):
    pass


def analyze_netloc(netloc):
    """Everything needed to bind a TCP socket from a parsed URL

    :return: family, host, port. port is ``None`` if the netloc does not
       specify it.
    """
    if netloc.startswith('['):
        h_end = netloc.find(']')
        if h_end < 0:
            raise ValueError(netloc)
        host = netloc[1:h_end]
        family = socket.AF_INET6

        if h_end + 1 == len(netloc):
            return family, host, None

        if netloc[h_end + 1] != ':':
            raise ValueError(netloc)
        return family, host, int(netloc[h_end + 2:])

    split = netloc.rsplit(':', 1)
    if len(split) == 1:
        host, port = netloc, None
    else:
        host, port = split
        port = int(port)

    # here actually I should let gai do the job
    # the thing is, we *must* do the same
    # as gRPC and I don't know yet what this is
    family = socket.AF_INET
    return family, host, port


def apply_default_port(netloc):
    """Return a netloc with default port applied if port wasn't specified."""
    family, host, port = analyze_netloc(netloc)
    if port is None:
        port = DEFAULT_TCP_PORT
    if family == socket.AF_INET6:
        return '[%s]:%d' % (host, port)
    return '%s:%d' % (host, port)



# Copyright 2020 Georges Racinet <georges.racinet@octobus.net>
#
# This software may be used and distributed according to the terms of the
# GNU General Public License version 2 or any later version.
#
# SPDX-License-Identifier: GPL-2.0-or-later
import grpc
import pytest

from hgitaly.oid import (
    blob_oid,
)
from hgitaly.revision import (
    ZERO_SHA_STR,
)
from hgitaly.service.blob import (
    iter_blob_chunks,
)
from hgitaly.stream import WRITE_BUFFER_SIZE

from hgitaly.tests.common import (
    make_empty_repo,
)

from hgitaly.stub.blob_pb2 import (
    GetBlobRequest,
    GetBlobResponse,
    GetBlobsRequest,
)
from hgitaly.stub.blob_pb2_grpc import BlobServiceStub


def test_get_blob(grpc_channel, server_repos_root):
    grpc_stub = BlobServiceStub(grpc_channel)
    wrapper, grpc_repo = make_empty_repo(server_repos_root)

    def do_rpc(oid, limit=-1):
        request = GetBlobRequest(repository=grpc_repo,
                                 oid=oid,
                                 limit=limit)
        return [resp for resp in grpc_stub.GetBlob(request)]

    small = wrapper.write_commit('foo', content='some content')
    oid = blob_oid(wrapper.repo, small.hex().decode(), b'foo')
    resps = do_rpc(oid)

    assert len(resps) == 1
    assert resps[0].data == b'some content'
    assert resps[0].size == 12
    assert resps[0].oid == oid

    resps = do_rpc(oid, limit=4)
    assert len(resps) == 1
    assert resps[0].data == b'some'
    assert resps[0].size == 12
    assert resps[0].oid == oid

    large_data = b'\xca\xfe' * 1024 * 65  # 1024 more than WRITE_BUFFER_SIZE
    large = wrapper.write_commit('foo', content=large_data, message='largebin')
    oid = blob_oid(wrapper.repo, large.hex().decode(), b'foo')
    resps = do_rpc(oid)

    assert len(resps) == 2
    assert resps[0].data == large_data[:WRITE_BUFFER_SIZE]
    assert resps[0].size == len(large_data)
    assert resps[0].oid == oid

    assert resps[1].data == b'\xca\xfe' * 1024
    assert resps[1].size == 0
    assert resps[1].oid == ''

    with pytest.raises(grpc.RpcError) as exc_info:
        do_rpc('')
    assert exc_info.value.code() == grpc.StatusCode.INVALID_ARGUMENT

    # unknown revision and path give a single, empty response
    for node_id, path in (('be56ef78' * 5, b'some/path'),
                          (ZERO_SHA_STR, b'some/path'),
                          (large.hex().decode(), b'no/such/file'),
                          ):
        assert do_rpc(
            blob_oid(wrapper.repo, node_id, path)
        ) == [GetBlobResponse()]


def test_get_blobs(grpc_channel, server_repos_root):
    grpc_stub = BlobServiceStub(grpc_channel)
    wrapper, grpc_repo = make_empty_repo(server_repos_root)
    repo = wrapper.repo
    RevisionPath = GetBlobsRequest.RevisionPath

    def do_rpc(rev_paths, limit=-1):
        request = GetBlobsRequest(
            repository=grpc_repo,
            revision_paths=[RevisionPath(revision=rev, path=path)
                            for rev, path in rev_paths],
            limit=limit)
        return [resp for resp in grpc_stub.GetBlobs(request)]

    wrapper.write_commit('foo', content='some content')
    two_files = wrapper.write_commit('bar', content='bar content')
    tip_hex = two_files.hex().decode()
    resps = do_rpc([(tip_hex, b'foo'),
                    (tip_hex, b'bar'),
                    (b'unknown-rev', b'foo'),
                    (tip_hex, b'unknown-file')
                    ])

    assert len(resps) == 4
    assert resps[0].data == b'some content'
    assert resps[0].size == 12
    assert resps[0].oid == blob_oid(repo, tip_hex, b'foo')

    assert resps[1].data == b'bar content'
    assert resps[1].size == 11
    assert resps[1].oid == blob_oid(repo, tip_hex, b'bar')

    assert not resps[2].oid
    assert not resps[3].oid

    resps = do_rpc([(tip_hex, b'foo'), (tip_hex, b'bar')], limit=4)
    assert len(resps) == 2
    assert resps[0].data == b'some'
    assert resps[0].size == 12
    assert resps[0].oid == blob_oid(repo, tip_hex, b'foo')

    assert resps[1].data == b'bar '
    assert resps[1].size == 11
    assert resps[1].oid == blob_oid(repo, tip_hex, b'bar')


def test_iter_blob_chunks():
    """Complements: edge cases etc."""

    assert list(iter_blob_chunks(b'aabbc', chunk_size=2)) == [
        (b'aa', True),
        (b'bb', False),
        (b'c', False),
    ]
    assert list(iter_blob_chunks(b'aabb', chunk_size=2)) == [
        (b'aa', True),
        (b'bb', False),
    ]
    assert list(iter_blob_chunks(b'')) == [(b'', True)]
    assert list(iter_blob_chunks(b'aabb', chunk_size=4)) == [(b'aabb', True)]



# Copyright 2021 Georges Racinet <georges.racinet@octobus.net>
#
# This software may be used and distributed according to the terms of the
# GNU General Public License version 2 or any later version.
#
# SPDX-License-Identifier: GPL-2.0-or-later
import attr
import os
import pwd
import socket
import subprocess

from . import TEST_DATA_DIR


@attr.s
class SSHServer:
    host_pub_key_path = attr.ib()
    host_priv_key_path = attr.ib()
    config_template_path = attr.ib()
    working_dir = attr.ib()
    authorized_keys_path = attr.ib()
    host_ipv4 = attr.ib(default='127.0.0.1')
    """IPv4 local address for the server to bind to.

    Domain names and IPv6 addresses are not supported (not even `::1`).

    Forcing the communication to be over IPv4 is lame, but it is the most
    reliable current way:
    - only one type of address avoids lots of complications which are not
      relevant for these tests.
    - there are contexts where IPv6 is not available, whereas those where
      the IPv4 loopback is not are excessively rare.
    """
    port = attr.ib(default=None)
    sshd_exe_path = attr.ib(default='/usr/sbin/sshd')
    initialized = False

    def init(self):
        self.user_name = pwd.getpwuid(os.getuid()).pw_name
        self.init_address()
        self.host = self.host_ipv4
        self.init_config()
        self.initialized = True

    def init_address(self):  # pragma no cover (depends on tests invocation)
        """Determine the port for the server to bind to.

        For best reliability in controlled environment such as CI,
        the port is read from the `HGITALY_TESTS_SSHD_PORT` environment
        variable.

        Otherwise, for developer convenience, we determine an available port.
        This is inherently racy, as the port may well not be available any
        more when the SSH server actually starts.
        """
        from_env = os.environ.get('HGITALY_TESTS_SSHD_PORT')
        if from_env is not None:
            self.port = int(from_env)
        else:
            sock = socket.create_server((self.host_ipv4, 0),
                                        family=socket.AF_INET)
            self.port = sock.getsockname()[1]
            sock.close()

    def init_config(self):
        config_path = self.sshd_config_path = self.working_dir / 'sshd_config'
        config_path.write_text(
            self.config_template_path.read_text().format(
                user=self.user_name,
                host=self.host,
                port=self.port,
                authorized_keys_path=self.authorized_keys_path),
            encoding='ascii')

        perms = 0o600  # not tracked by hg, won't look as a file modification
        self.host_pub_key_path.chmod(perms)
        self.host_priv_key_path.chmod(perms)

    def start(self):
        self.process = subprocess.Popen((self.sshd_exe_path,
                                         '-h', self.host_priv_key_path,
                                         '-c', self.host_pub_key_path,
                                         '-f', self.sshd_config_path,
                                         '-D', '-e',
                                         ),
                                        )

    def stop(self):
        self.process.terminate()
        self.process.wait()


def hg_ssh_setup(working_dir):
    server = SSHServer(
        host_pub_key_path=TEST_DATA_DIR / 'ssh_host_ecdsa_key.pub',
        host_priv_key_path=TEST_DATA_DIR / 'ssh_host_ecdsa_key',
        config_template_path=TEST_DATA_DIR / 'sshd_config',
        authorized_keys_path=TEST_DATA_DIR / 'authorized_keys',
        working_dir=working_dir,
    )
    server.init()
    client_key = (TEST_DATA_DIR / 'id_ecdsa_user').read_text()
    known_hosts = (TEST_DATA_DIR / 'known_hosts').read_text().format(
        host=server.host, port=server.port)
    server.start()
    yield server, client_key, known_hosts

    server.stop()



# Copyright 2020 Georges Racinet <georges.racinet@octobus.net>
#
# This software may be used and distributed according to the terms of the
# GNU General Public License version 2 or any later version.
#
# SPDX-License-Identifier: GPL-2.0-or-later
import itertools

from .stub.shared_pb2 import (
    PaginationCursor,
)

# default as in Gitaly's internal/helper/chunk/chunker.go
DEFAULT_CHUNK_SIZE = 20


def chunked(iterable,
            size=DEFAULT_CHUNK_SIZE,
            limit=None,
            force_if_empty=False):
    """Return a generator of chunks out of given iterable.

    >>> [chunk for chunk in chunked(range(3), size=2)]
    [[0, 1], [2]]
    >>> [chunk for chunk in chunked(range(4), size=2)]
    [[0, 1], [2, 3]]

    :param limit: maximum total number of results (unlimited if evaluates to
                  ``False``

    >>> [chunk for chunk in chunked(range(7), size=2, limit=2)]
    [[0, 1]]
    """
    chunk = []
    if limit:
        iterable = itertools.islice(iterable, 0, limit)
    for i, val in enumerate(iterable):
        if i != 0 and i % size == 0:
            yield chunk
            chunk = []
        chunk.append(val)

    if chunk or force_if_empty:
        yield chunk


def chunked_with_cursor(message_class, iterable, next_cursor, builder,
                        size=DEFAULT_CHUNK_SIZE, limit=None):
    """Iterate in chunks and generate gRPC messages with pagination cursor.

    It is important that the pagination cursor is present only on the
    first message. Using the default value `PaginationCursor()` instead can be
    a breaking mistake (see hgitaly#91 for the gory details).

    This helper is there to help do the right thing, not to make it
    especially simpler for the caller.
    """
    first = True
    for chunk in chunked(iterable):
        fields = builder(chunk)
        if first:
            fields['pagination_cursor'] = PaginationCursor(
                next_cursor=next_cursor)
            first = False
        yield message_class(**fields)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: mercurial-changeset.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from . import lint_pb2 as lint__pb2
from . import shared_pb2 as shared__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x19mercurial-changeset.proto\x12\x07hgitaly\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\nlint.proto\x1a\x0cshared.proto\"\x9b\x02\n\x12MercurialChangeset\x12\n\n\x02id\x18\x01 \x01(\t\x12\x12\n\nparent_ids\x18\x02 \x03(\t\x12\r\n\x05title\x18\x03 \x01(\x0c\x12\x13\n\x0b\x64\x65scription\x18\x04 \x01(\x0c\x12\x0e\n\x06\x61uthor\x18\x05 \x01(\x0c\x12(\n\x04\x64\x61te\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0e\n\x06\x62ranch\x18\x07 \x01(\x0c\x12\r\n\x05topic\x18\x08 \x01(\x0c\x12\x10\n\x08obsolete\x18\t \x01(\x08\x12\x0c\n\x04\x66qbn\x18\n \x01(\x0c\x12\x17\n\x0ftopic_namespace\x18\x0b \x01(\x0c\x12/\n\x05phase\x18\x0c \x01(\x0e\x32 .hgitaly.MercurialChangesetPhase\"\xc0\x01\n\x1eListMercurialChangesetsRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12.\n\x04view\x18\x02 \x01(\x0e\x32 .hgitaly.MercurialRepositoryView\x12\x0e\n\x06revset\x18\x03 \x01(\x0c\x12\x30\n\x06\x66ields\x18\x04 \x03(\x0e\x32 .hgitaly.MercurialChangesetField\"R\n\x1fListMercurialChangesetsResponse\x12/\n\nchangesets\x18\x01 \x03(\x0b\x32\x1b.hgitaly.MercurialChangeset*X\n\x17MercurialChangesetPhase\x12\n\n\x06PUBLIC\x10\x00\x12\t\n\x05\x44RAFT\x10\x01\x12\n\n\x06SECRET\x10\x02\x12\x0c\n\x08\x41RCHIVED\x10 \x12\x0c\n\x08INTERNAL\x10`*\xb3\x01\n\x17MercurialChangesetField\x12\x07\n\x03\x41LL\x10\x00\x12\x0e\n\nPARENT_IDS\x10\x01\x12\t\n\x05TITLE\x10\x02\x12\x0f\n\x0b\x44\x45SCRIPTION\x10\x03\x12\n\n\x06\x41UTHOR\x10\x04\x12\x08\n\x04\x44\x41TE\x10\x05\x12\n\n\x06\x42RANCH\x10\x06\x12\t\n\x05TOPIC\x10\x07\x12\x0c\n\x08OBSOLETE\x10\x08\x12\x08\n\x04\x46QBN\x10\t\x12\x13\n\x0fTOPIC_NAMESPACE\x10\n\x12\t\n\x05PHASE\x10\x0b*6\n\x17MercurialRepositoryView\x12\x0b\n\x07VISIBLE\x10\x00\x12\x0e\n\nUNFILTERED\x10\x01\x32\x93\x01\n\x19MercurialChangesetService\x12v\n\x17ListMercurialChangesets\x12\'.hgitaly.ListMercurialChangesetsRequest\x1a(.hgitaly.ListMercurialChangesetsResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'mercurial_changeset_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _LISTMERCURIALCHANGESETSREQUEST.fields_by_name['repository']._options = None
  _LISTMERCURIALCHANGESETSREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _MERCURIALCHANGESETSERVICE.methods_by_name['ListMercurialChangesets']._options = None
  _MERCURIALCHANGESETSERVICE.methods_by_name['ListMercurialChangesets']._serialized_options = b'\372\227(\002\010\002'
  _globals['_MERCURIALCHANGESETPHASE']._serialized_start=662
  _globals['_MERCURIALCHANGESETPHASE']._serialized_end=750
  _globals['_MERCURIALCHANGESETFIELD']._serialized_start=753
  _globals['_MERCURIALCHANGESETFIELD']._serialized_end=932
  _globals['_MERCURIALREPOSITORYVIEW']._serialized_start=934
  _globals['_MERCURIALREPOSITORYVIEW']._serialized_end=988
  _globals['_MERCURIALCHANGESET']._serialized_start=98
  _globals['_MERCURIALCHANGESET']._serialized_end=381
  _globals['_LISTMERCURIALCHANGESETSREQUEST']._serialized_start=384
  _globals['_LISTMERCURIALCHANGESETSREQUEST']._serialized_end=576
  _globals['_LISTMERCURIALCHANGESETSRESPONSE']._serialized_start=578
  _globals['_LISTMERCURIALCHANGESETSRESPONSE']._serialized_end=660
  _globals['_MERCURIALCHANGESETSERVICE']._serialized_start=991
  _globals['_MERCURIALCHANGESETSERVICE']._serialized_end=1138
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ref.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import errors_pb2 as errors__pb2
from . import lint_pb2 as lint__pb2
from . import shared_pb2 as shared__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\tref.proto\x12\x06gitaly\x1a\x0c\x65rrors.proto\x1a\nlint.proto\x1a\x0cshared.proto\"_\n\x1c\x46indDefaultBranchNameRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x11\n\thead_only\x18\x02 \x01(\x08\"-\n\x1d\x46indDefaultBranchNameResponse\x12\x0c\n\x04name\x18\x01 \x01(\x0c\"\xf1\x01\n\x18\x46indLocalBranchesRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x38\n\x07sort_by\x18\x02 \x01(\x0e\x32\'.gitaly.FindLocalBranchesRequest.SortBy\x12\x36\n\x11pagination_params\x18\x03 \x01(\x0b\x32\x1b.gitaly.PaginationParameter\"5\n\x06SortBy\x12\x08\n\x04NAME\x10\x00\x12\x0f\n\x0bUPDATED_ASC\x10\x01\x12\x10\n\x0cUPDATED_DESC\x10\x02\"S\n\x19\x46indLocalBranchesResponse\x12&\n\x0elocal_branches\x18\x02 \x03(\x0b\x32\x0e.gitaly.BranchJ\x04\x08\x01\x10\x02R\x08\x62ranches\"t\n\x16\x46indAllBranchesRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x13\n\x0bmerged_only\x18\x02 \x01(\x08\x12\x17\n\x0fmerged_branches\x18\x03 \x03(\x0c\"\x8e\x01\n\x17\x46indAllBranchesResponse\x12\x38\n\x08\x62ranches\x18\x01 \x03(\x0b\x32&.gitaly.FindAllBranchesResponse.Branch\x1a\x39\n\x06\x42ranch\x12\x0c\n\x04name\x18\x01 \x01(\x0c\x12!\n\x06target\x18\x02 \x01(\x0b\x32\x11.gitaly.GitCommit\"P\n\x0e\x46indTagRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x10\n\x08tag_name\x18\x02 \x01(\x0c\"+\n\x0f\x46indTagResponse\x12\x18\n\x03tag\x18\x01 \x01(\x0b\x32\x0b.gitaly.Tag\"P\n\x0c\x46indTagError\x12\x37\n\rtag_not_found\x18\x01 \x01(\x0b\x32\x1e.gitaly.ReferenceNotFoundErrorH\x00\x42\x07\n\x05\x65rror\"\xd1\x02\n\x12\x46indAllTagsRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x32\n\x07sort_by\x18\x02 \x01(\x0b\x32!.gitaly.FindAllTagsRequest.SortBy\x12\x36\n\x11pagination_params\x18\x03 \x01(\x0b\x32\x1b.gitaly.PaginationParameter\x1a\xa0\x01\n\x06SortBy\x12\x32\n\x03key\x18\x01 \x01(\x0e\x32%.gitaly.FindAllTagsRequest.SortBy.Key\x12(\n\tdirection\x18\x02 \x01(\x0e\x32\x15.gitaly.SortDirection\"8\n\x03Key\x12\x0b\n\x07REFNAME\x10\x00\x12\x0f\n\x0b\x43REATORDATE\x10\x01\x12\x13\n\x0fVERSION_REFNAME\x10\x02\"0\n\x13\x46indAllTagsResponse\x12\x19\n\x04tags\x18\x01 \x03(\x0b\x32\x0b.gitaly.Tag\"M\n\x10RefExistsRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x0b\n\x03ref\x18\x02 \x01(\x0c\"\"\n\x11RefExistsResponse\x12\r\n\x05value\x18\x01 \x01(\x08\"O\n\x11\x46indBranchRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x0c\n\x04name\x18\x02 \x01(\x0c\"4\n\x12\x46indBranchResponse\x12\x1e\n\x06\x62ranch\x18\x01 \x01(\x0b\x32\x0e.gitaly.Branch\"\xcb\x01\n\x17UpdateReferencesRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x37\n\x07updates\x18\x02 \x03(\x0b\x32&.gitaly.UpdateReferencesRequest.Update\x1aI\n\x06Update\x12\x11\n\treference\x18\x01 \x01(\x0c\x12\x15\n\rold_object_id\x18\x02 \x01(\x0c\x12\x15\n\rnew_object_id\x18\x03 \x01(\x0c\"\x1a\n\x18UpdateReferencesResponse\"\xde\x01\n\x15UpdateReferencesError\x12\x37\n\x0einvalid_format\x18\x01 \x01(\x0b\x32\x1d.gitaly.InvalidRefFormatErrorH\x00\x12:\n\x11references_locked\x18\x02 \x01(\x0b\x32\x1d.gitaly.ReferencesLockedErrorH\x00\x12G\n\x18reference_state_mismatch\x18\x03 \x01(\x0b\x32#.gitaly.ReferenceStateMismatchErrorH\x00\x42\x07\n\x05\x65rror\"k\n\x11\x44\x65leteRefsRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x1a\n\x12\x65xcept_with_prefix\x18\x02 \x03(\x0c\x12\x0c\n\x04refs\x18\x03 \x03(\x0c\"\'\n\x12\x44\x65leteRefsResponse\x12\x11\n\tgit_error\x18\x01 \x01(\t\"\x8f\x01\n\x0f\x44\x65leteRefsError\x12\x37\n\x0einvalid_format\x18\x01 \x01(\x0b\x32\x1d.gitaly.InvalidRefFormatErrorH\x00\x12:\n\x11references_locked\x18\x02 \x01(\x0b\x32\x1d.gitaly.ReferencesLockedErrorH\x00\x42\x07\n\x05\x65rror\"x\n&ListBranchNamesContainingCommitRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x11\n\tcommit_id\x18\x02 \x01(\t\x12\r\n\x05limit\x18\x03 \x01(\r\"E\n\'ListBranchNamesContainingCommitResponse\x12\x14\n\x0c\x62ranch_names\x18\x02 \x03(\x0cJ\x04\x08\x01\x10\x02\"u\n#ListTagNamesContainingCommitRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x11\n\tcommit_id\x18\x02 \x01(\t\x12\r\n\x05limit\x18\x03 \x01(\r\"?\n$ListTagNamesContainingCommitResponse\x12\x11\n\ttag_names\x18\x02 \x03(\x0cJ\x04\x08\x01\x10\x02\"^\n\x17GetTagSignaturesRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x15\n\rtag_revisions\x18\x02 \x03(\t\"\xa1\x01\n\x18GetTagSignaturesResponse\x12\x41\n\nsignatures\x18\x01 \x03(\x0b\x32-.gitaly.GetTagSignaturesResponse.TagSignature\x1a\x42\n\x0cTagSignature\x12\x0e\n\x06tag_id\x18\x01 \x01(\t\x12\x11\n\tsignature\x18\x02 \x01(\x0c\x12\x0f\n\x07\x63ontent\x18\x03 \x01(\x0c\"g\n\x15GetTagMessagesRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x0f\n\x07tag_ids\x18\x03 \x03(\tJ\x04\x08\x02\x10\x03R\ttag_names\"I\n\x16GetTagMessagesResponse\x12\x0f\n\x07message\x18\x02 \x01(\x0c\x12\x0e\n\x06tag_id\x18\x03 \x01(\tJ\x04\x08\x01\x10\x02R\x08tag_name\"a\n\x1c\x46indAllRemoteBranchesRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x13\n\x0bremote_name\x18\x02 \x01(\t\"A\n\x1d\x46indAllRemoteBranchesResponse\x12 \n\x08\x62ranches\x18\x01 \x03(\x0b\x32\x0e.gitaly.Branch\"\xeb\x02\n\x0fListRefsRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x10\n\x08patterns\x18\x02 \x03(\x0c\x12\x0c\n\x04head\x18\x03 \x01(\x08\x12/\n\x07sort_by\x18\x04 \x01(\x0b\x32\x1e.gitaly.ListRefsRequest.SortBy\x12\x18\n\x10pointing_at_oids\x18\x05 \x03(\x0c\x12\x11\n\tpeel_tags\x18\x06 \x01(\x08\x1a\xab\x01\n\x06SortBy\x12/\n\x03key\x18\x01 \x01(\x0e\x32\".gitaly.ListRefsRequest.SortBy.Key\x12(\n\tdirection\x18\x02 \x01(\x0e\x32\x15.gitaly.SortDirection\"F\n\x03Key\x12\x0b\n\x07REFNAME\x10\x00\x12\x0f\n\x0b\x43REATORDATE\x10\x01\x12\x0e\n\nAUTHORDATE\x10\x02\x12\x11\n\rCOMMITTERDATE\x10\x03\"\x8c\x01\n\x10ListRefsResponse\x12\x36\n\nreferences\x18\x01 \x03(\x0b\x32\".gitaly.ListRefsResponse.Reference\x1a@\n\tReference\x12\x0c\n\x04name\x18\x01 \x01(\x0c\x12\x0e\n\x06target\x18\x02 \x01(\t\x12\x15\n\rpeeled_target\x18\x03 \x01(\t\"\x8a\x01\n\x14\x46indRefsByOIDRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x0b\n\x03oid\x18\x02 \x01(\t\x12\x14\n\x0cref_patterns\x18\x03 \x03(\t\x12\x12\n\nsort_field\x18\x04 \x01(\t\x12\r\n\x05limit\x18\x05 \x01(\r\"%\n\x15\x46indRefsByOIDResponse\x12\x0c\n\x04refs\x18\x01 \x03(\t2\xf7\x0b\n\nRefService\x12l\n\x15\x46indDefaultBranchName\x12$.gitaly.FindDefaultBranchNameRequest\x1a%.gitaly.FindDefaultBranchNameResponse\"\x06\xfa\x97(\x02\x08\x02\x12\x62\n\x11\x46indLocalBranches\x12 .gitaly.FindLocalBranchesRequest\x1a!.gitaly.FindLocalBranchesResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12\\\n\x0f\x46indAllBranches\x12\x1e.gitaly.FindAllBranchesRequest\x1a\x1f.gitaly.FindAllBranchesResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12P\n\x0b\x46indAllTags\x12\x1a.gitaly.FindAllTagsRequest\x1a\x1b.gitaly.FindAllTagsResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12\x42\n\x07\x46indTag\x12\x16.gitaly.FindTagRequest\x1a\x17.gitaly.FindTagResponse\"\x06\xfa\x97(\x02\x08\x02\x12n\n\x15\x46indAllRemoteBranches\x12$.gitaly.FindAllRemoteBranchesRequest\x1a%.gitaly.FindAllRemoteBranchesResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12H\n\tRefExists\x12\x18.gitaly.RefExistsRequest\x1a\x19.gitaly.RefExistsResponse\"\x06\xfa\x97(\x02\x08\x02\x12K\n\nFindBranch\x12\x19.gitaly.FindBranchRequest\x1a\x1a.gitaly.FindBranchResponse\"\x06\xfa\x97(\x02\x08\x02\x12_\n\x10UpdateReferences\x12\x1f.gitaly.UpdateReferencesRequest\x1a .gitaly.UpdateReferencesResponse\"\x06\xfa\x97(\x02\x08\x01(\x01\x12K\n\nDeleteRefs\x12\x19.gitaly.DeleteRefsRequest\x1a\x1a.gitaly.DeleteRefsResponse\"\x06\xfa\x97(\x02\x08\x01\x12\x8c\x01\n\x1fListBranchNamesContainingCommit\x12..gitaly.ListBranchNamesContainingCommitRequest\x1a/.gitaly.ListBranchNamesContainingCommitResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12\x83\x01\n\x1cListTagNamesContainingCommit\x12+.gitaly.ListTagNamesContainingCommitRequest\x1a,.gitaly.ListTagNamesContainingCommitResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12_\n\x10GetTagSignatures\x12\x1f.gitaly.GetTagSignaturesRequest\x1a .gitaly.GetTagSignaturesResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12Y\n\x0eGetTagMessages\x12\x1d.gitaly.GetTagMessagesRequest\x1a\x1e.gitaly.GetTagMessagesResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12G\n\x08ListRefs\x12\x17.gitaly.ListRefsRequest\x1a\x18.gitaly.ListRefsResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12T\n\rFindRefsByOID\x12\x1c.gitaly.FindRefsByOIDRequest\x1a\x1d.gitaly.FindRefsByOIDResponse\"\x06\xfa\x97(\x02\x08\x02\x42\x34Z2gitlab.com/gitlab-org/gitaly/v16/proto/go/gitalypbb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ref_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z2gitlab.com/gitlab-org/gitaly/v16/proto/go/gitalypb'
  _FINDDEFAULTBRANCHNAMEREQUEST.fields_by_name['repository']._options = None
  _FINDDEFAULTBRANCHNAMEREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _FINDLOCALBRANCHESREQUEST.fields_by_name['repository']._options = None
  _FINDLOCALBRANCHESREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _FINDALLBRANCHESREQUEST.fields_by_name['repository']._options = None
  _FINDALLBRANCHESREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _FINDTAGREQUEST.fields_by_name['repository']._options = None
  _FINDTAGREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _FINDALLTAGSREQUEST.fields_by_name['repository']._options = None
  _FINDALLTAGSREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _REFEXISTSREQUEST.fields_by_name['repository']._options = None
  _REFEXISTSREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _FINDBRANCHREQUEST.fields_by_name['repository']._options = None
  _FINDBRANCHREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _UPDATEREFERENCESREQUEST.fields_by_name['repository']._options = None
  _UPDATEREFERENCESREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _DELETEREFSREQUEST.fields_by_name['repository']._options = None
  _DELETEREFSREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _LISTBRANCHNAMESCONTAININGCOMMITREQUEST.fields_by_name['repository']._options = None
  _LISTBRANCHNAMESCONTAININGCOMMITREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _LISTTAGNAMESCONTAININGCOMMITREQUEST.fields_by_name['repository']._options = None
  _LISTTAGNAMESCONTAININGCOMMITREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _GETTAGSIGNATURESREQUEST.fields_by_name['repository']._options = None
  _GETTAGSIGNATURESREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _GETTAGMESSAGESREQUEST.fields_by_name['repository']._options = None
  _GETTAGMESSAGESREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _FINDALLREMOTEBRANCHESREQUEST.fields_by_name['repository']._options = None
  _FINDALLREMOTEBRANCHESREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _LISTREFSREQUEST.fields_by_name['repository']._options = None
  _LISTREFSREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _FINDREFSBYOIDREQUEST.fields_by_name['repository']._options = None
  _FINDREFSBYOIDREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _REFSERVICE.methods_by_name['FindDefaultBranchName']._options = None
  _REFSERVICE.methods_by_name['FindDefaultBranchName']._serialized_options = b'\372\227(\002\010\002'
  _REFSERVICE.methods_by_name['FindLocalBranches']._options = None
  _REFSERVICE.methods_by_name['FindLocalBranches']._serialized_options = b'\372\227(\002\010\002'
  _REFSERVICE.methods_by_name['FindAllBranches']._options = None
  _REFSERVICE.methods_by_name['FindAllBranches']._serialized_options = b'\372\227(\002\010\002'
  _REFSERVICE.methods_by_name['FindAllTags']._options = None
  _REFSERVICE.methods_by_name['FindAllTags']._serialized_options = b'\372\227(\002\010\002'
  _REFSERVICE.methods_by_name['FindTag']._options = None
  _REFSERVICE.methods_by_name['FindTag']._serialized_options = b'\372\227(\002\010\002'
  _REFSERVICE.methods_by_name['FindAllRemoteBranches']._options = None
  _REFSERVICE.methods_by_name['FindAllRemoteBranches']._serialized_options = b'\372\227(\002\010\002'
  _REFSERVICE.methods_by_name['RefExists']._options = None
  _REFSERVICE.methods_by_name['RefExists']._serialized_options = b'\372\227(\002\010\002'
  _REFSERVICE.methods_by_name['FindBranch']._options = None
  _REFSERVICE.methods_by_name['FindBranch']._serialized_options = b'\372\227(\002\010\002'
  _REFSERVICE.methods_by_name['UpdateReferences']._options = None
  _REFSERVICE.methods_by_name['UpdateReferences']._serialized_options = b'\372\227(\002\010\001'
  _REFSERVICE.methods_by_name['DeleteRefs']._options = None
  _REFSERVICE.methods_by_name['DeleteRefs']._serialized_options = b'\372\227(\002\010\001'
  _REFSERVICE.methods_by_name['ListBranchNamesContainingCommit']._options = None
  _REFSERVICE.methods_by_name['ListBranchNamesContainingCommit']._serialized_options = b'\372\227(\002\010\002'
  _REFSERVICE.methods_by_name['ListTagNamesContainingCommit']._options = None
  _REFSERVICE.methods_by_name['ListTagNamesContainingCommit']._serialized_options = b'\372\227(\002\010\002'
  _REFSERVICE.methods_by_name['GetTagSignatures']._options = None
  _REFSERVICE.methods_by_name['GetTagSignatures']._serialized_options = b'\372\227(\002\010\002'
  _REFSERVICE.methods_by_name['GetTagMessages']._options = None
  _REFSERVICE.methods_by_name['GetTagMessages']._serialized_options = b'\372\227(\002\010\002'
  _REFSERVICE.methods_by_name['ListRefs']._options = None
  _REFSERVICE.methods_by_name['ListRefs']._serialized_options = b'\372\227(\002\010\002'
  _REFSERVICE.methods_by_name['FindRefsByOID']._options = None
  _REFSERVICE.methods_by_name['FindRefsByOID']._serialized_options = b'\372\227(\002\010\002'
  _globals['_FINDDEFAULTBRANCHNAMEREQUEST']._serialized_start=61
  _globals['_FINDDEFAULTBRANCHNAMEREQUEST']._serialized_end=156
  _globals['_FINDDEFAULTBRANCHNAMERESPONSE']._serialized_start=158
  _globals['_FINDDEFAULTBRANCHNAMERESPONSE']._serialized_end=203
  _globals['_FINDLOCALBRANCHESREQUEST']._serialized_start=206
  _globals['_FINDLOCALBRANCHESREQUEST']._serialized_end=447
  _globals['_FINDLOCALBRANCHESREQUEST_SORTBY']._serialized_start=394
  _globals['_FINDLOCALBRANCHESREQUEST_SORTBY']._serialized_end=447
  _globals['_FINDLOCALBRANCHESRESPONSE']._serialized_start=449
  _globals['_FINDLOCALBRANCHESRESPONSE']._serialized_end=532
  _globals['_FINDALLBRANCHESREQUEST']._serialized_start=534
  _globals['_FINDALLBRANCHESREQUEST']._serialized_end=650
  _globals['_FINDALLBRANCHESRESPONSE']._serialized_start=653
  _globals['_FINDALLBRANCHESRESPONSE']._serialized_end=795
  _globals['_FINDALLBRANCHESRESPONSE_BRANCH']._serialized_start=738
  _globals['_FINDALLBRANCHESRESPONSE_BRANCH']._serialized_end=795
  _globals['_FINDTAGREQUEST']._serialized_start=797
  _globals['_FINDTAGREQUEST']._serialized_end=877
  _globals['_FINDTAGRESPONSE']._serialized_start=879
  _globals['_FINDTAGRESPONSE']._serialized_end=922
  _globals['_FINDTAGERROR']._serialized_start=924
  _globals['_FINDTAGERROR']._serialized_end=1004
  _globals['_FINDALLTAGSREQUEST']._serialized_start=1007
  _globals['_FINDALLTAGSREQUEST']._serialized_end=1344
  _globals['_FINDALLTAGSREQUEST_SORTBY']._serialized_start=1184
  _globals['_FINDALLTAGSREQUEST_SORTBY']._serialized_end=1344
  _globals['_FINDALLTAGSREQUEST_SORTBY_KEY']._serialized_start=1288
  _globals['_FINDALLTAGSREQUEST_SORTBY_KEY']._serialized_end=1344
  _globals['_FINDALLTAGSRESPONSE']._serialized_start=1346
  _globals['_FINDALLTAGSRESPONSE']._serialized_end=1394
  _globals['_REFEXISTSREQUEST']._serialized_start=1396
  _globals['_REFEXISTSREQUEST']._serialized_end=1473
  _globals['_REFEXISTSRESPONSE']._serialized_start=1475
  _globals['_REFEXISTSRESPONSE']._serialized_end=1509
  _globals['_FINDBRANCHREQUEST']._serialized_start=1511
  _globals['_FINDBRANCHREQUEST']._serialized_end=1590
  _globals['_FINDBRANCHRESPONSE']._serialized_start=1592
  _globals['_FINDBRANCHRESPONSE']._serialized_end=1644
  _globals['_UPDATEREFERENCESREQUEST']._serialized_start=1647
  _globals['_UPDATEREFERENCESREQUEST']._serialized_end=1850
  _globals['_UPDATEREFERENCESREQUEST_UPDATE']._serialized_start=1777
  _globals['_UPDATEREFERENCESREQUEST_UPDATE']._serialized_end=1850
  _globals['_UPDATEREFERENCESRESPONSE']._serialized_start=1852
  _globals['_UPDATEREFERENCESRESPONSE']._serialized_end=1878
  _globals['_UPDATEREFERENCESERROR']._serialized_start=1881
  _globals['_UPDATEREFERENCESERROR']._serialized_end=2103
  _globals['_DELETEREFSREQUEST']._serialized_start=2105
  _globals['_DELETEREFSREQUEST']._serialized_end=2212
  _globals['_DELETEREFSRESPONSE']._serialized_start=2214
  _globals['_DELETEREFSRESPONSE']._serialized_end=2253
  _globals['_DELETEREFSERROR']._serialized_start=2256
  _globals['_DELETEREFSERROR']._serialized_end=2399
  _globals['_LISTBRANCHNAMESCONTAININGCOMMITREQUEST']._serialized_start=2401
  _globals['_LISTBRANCHNAMESCONTAININGCOMMITREQUEST']._serialized_end=2521
  _globals['_LISTBRANCHNAMESCONTAININGCOMMITRESPONSE']._serialized_start=2523
  _globals['_LISTBRANCHNAMESCONTAININGCOMMITRESPONSE']._serialized_end=2592
  _globals['_LISTTAGNAMESCONTAININGCOMMITREQUEST']._serialized_start=2594
  _globals['_LISTTAGNAMESCONTAININGCOMMITREQUEST']._serialized_end=2711
  _globals['_LISTTAGNAMESCONTAININGCOMMITRESPONSE']._serialized_start=2713
  _globals['_LISTTAGNAMESCONTAININGCOMMITRESPONSE']._serialized_end=2776
  _globals['_GETTAGSIGNATURESREQUEST']._serialized_start=2778
  _globals['_GETTAGSIGNATURESREQUEST']._serialized_end=2872
  _globals['_GETTAGSIGNATURESRESPONSE']._serialized_start=2875
  _globals['_GETTAGSIGNATURESRESPONSE']._serialized_end=3036
  _globals['_GETTAGSIGNATURESRESPONSE_TAGSIGNATURE']._serialized_start=2970
  _globals['_GETTAGSIGNATURESRESPONSE_TAGSIGNATURE']._serialized_end=3036
  _globals['_GETTAGMESSAGESREQUEST']._serialized_start=3038
  _globals['_GETTAGMESSAGESREQUEST']._serialized_end=3141
  _globals['_GETTAGMESSAGESRESPONSE']._serialized_start=3143
  _globals['_GETTAGMESSAGESRESPONSE']._serialized_end=3216
  _globals['_FINDALLREMOTEBRANCHESREQUEST']._serialized_start=3218
  _globals['_FINDALLREMOTEBRANCHESREQUEST']._serialized_end=3315
  _globals['_FINDALLREMOTEBRANCHESRESPONSE']._serialized_start=3317
  _globals['_FINDALLREMOTEBRANCHESRESPONSE']._serialized_end=3382
  _globals['_LISTREFSREQUEST']._serialized_start=3385
  _globals['_LISTREFSREQUEST']._serialized_end=3748
  _globals['_LISTREFSREQUEST_SORTBY']._serialized_start=3577
  _globals['_LISTREFSREQUEST_SORTBY']._serialized_end=3748
  _globals['_LISTREFSREQUEST_SORTBY_KEY']._serialized_start=3678
  _globals['_LISTREFSREQUEST_SORTBY_KEY']._serialized_end=3748
  _globals['_LISTREFSRESPONSE']._serialized_start=3751
  _globals['_LISTREFSRESPONSE']._serialized_end=3891
  _globals['_LISTREFSRESPONSE_REFERENCE']._serialized_start=3827
  _globals['_LISTREFSRESPONSE_REFERENCE']._serialized_end=3891
  _globals['_FINDREFSBYOIDREQUEST']._serialized_start=3894
  _globals['_FINDREFSBYOIDREQUEST']._serialized_end=4032
  _globals['_FINDREFSBYOIDRESPONSE']._serialized_start=4034
  _globals['_FINDREFSBYOIDRESPONSE']._serialized_end=4071
  _globals['_REFSERVICE']._serialized_start=4074
  _globals['_REFSERVICE']._serialized_end=5601
# @@protoc_insertion_point(module_scope)



# Copyright 2020 Georges Racinet <georges.racinet@octobus.net>
#
# This software may be used and distributed according to the terms of the
# GNU General Public License version 2 or any later version.
#
# SPDX-License-Identifier: GPL-2.0-or-later
from socket import (
    AF_INET,
    AF_INET6,
)
import pytest

from ..address import (
    DEFAULT_TCP_PORT,
    analyze_netloc,
    apply_default_port,
)


def test_analyze_netloc():
    analyze = analyze_netloc

    assert analyze('127.1.2.3') == (AF_INET, '127.1.2.3', None)
    assert analyze('127.1.2.3:66') == (AF_INET, '127.1.2.3', 66)

    for ip6 in ('::1', '2001:db8:cafe::1', '2001:db8:1:2:3:4:5:6'):
        assert analyze('[%s]' % ip6) == (AF_INET6, ip6, None)
        assert analyze('[%s]:66' % ip6) == (AF_INET6, ip6, 66)

    with pytest.raises(ValueError) as exc_info:
        analyze('[::1')
    assert exc_info.value.args == ('[::1', )

    with pytest.raises(ValueError) as exc_info:
        analyze('[::]foo')
    assert exc_info.value.args == ('[::]foo', )


def test_apply_default_port():
    adp = apply_default_port
    for host in ('localhost', '[::1]', '127.0.0.1'):
        assert adp(host + ':123') == host + ':123'
        assert adp(host) == host + ':' + str(DEFAULT_TCP_PORT)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import operations_pb2 as operations__pb2


class OperationServiceStub(object):
    """OperationService provides an interface for performing mutating git
    operations on a repository on behalf of a user. The user's operation is
    treated as untrusted. Any reference update is thus checked against GitLab's
    '/allowed' endpoint.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.UserCreateBranch = channel.unary_unary(
                '/gitaly.OperationService/UserCreateBranch',
                request_serializer=operations__pb2.UserCreateBranchRequest.SerializeToString,
                response_deserializer=operations__pb2.UserCreateBranchResponse.FromString,
                )
        self.UserUpdateBranch = channel.unary_unary(
                '/gitaly.OperationService/UserUpdateBranch',
                request_serializer=operations__pb2.UserUpdateBranchRequest.SerializeToString,
                response_deserializer=operations__pb2.UserUpdateBranchResponse.FromString,
                )
        self.UserDeleteBranch = channel.unary_unary(
                '/gitaly.OperationService/UserDeleteBranch',
                request_serializer=operations__pb2.UserDeleteBranchRequest.SerializeToString,
                response_deserializer=operations__pb2.UserDeleteBranchResponse.FromString,
                )
        self.UserCreateTag = channel.unary_unary(
                '/gitaly.OperationService/UserCreateTag',
                request_serializer=operations__pb2.UserCreateTagRequest.SerializeToString,
                response_deserializer=operations__pb2.UserCreateTagResponse.FromString,
                )
        self.UserDeleteTag = channel.unary_unary(
                '/gitaly.OperationService/UserDeleteTag',
                request_serializer=operations__pb2.UserDeleteTagRequest.SerializeToString,
                response_deserializer=operations__pb2.UserDeleteTagResponse.FromString,
                )
        self.UserMergeToRef = channel.unary_unary(
                '/gitaly.OperationService/UserMergeToRef',
                request_serializer=operations__pb2.UserMergeToRefRequest.SerializeToString,
                response_deserializer=operations__pb2.UserMergeToRefResponse.FromString,
                )
        self.UserRebaseToRef = channel.unary_unary(
                '/gitaly.OperationService/UserRebaseToRef',
                request_serializer=operations__pb2.UserRebaseToRefRequest.SerializeToString,
                response_deserializer=operations__pb2.UserRebaseToRefResponse.FromString,
                )
        self.UserMergeBranch = channel.stream_stream(
                '/gitaly.OperationService/UserMergeBranch',
                request_serializer=operations__pb2.UserMergeBranchRequest.SerializeToString,
                response_deserializer=operations__pb2.UserMergeBranchResponse.FromString,
                )
        self.UserFFBranch = channel.unary_unary(
                '/gitaly.OperationService/UserFFBranch',
                request_serializer=operations__pb2.UserFFBranchRequest.SerializeToString,
                response_deserializer=operations__pb2.UserFFBranchResponse.FromString,
                )
        self.UserCherryPick = channel.unary_unary(
                '/gitaly.OperationService/UserCherryPick',
                request_serializer=operations__pb2.UserCherryPickRequest.SerializeToString,
                response_deserializer=operations__pb2.UserCherryPickResponse.FromString,
                )
        self.UserCommitFiles = channel.stream_unary(
                '/gitaly.OperationService/UserCommitFiles',
                request_serializer=operations__pb2.UserCommitFilesRequest.SerializeToString,
                response_deserializer=operations__pb2.UserCommitFilesResponse.FromString,
                )
        self.UserRebaseConfirmable = channel.stream_stream(
                '/gitaly.OperationService/UserRebaseConfirmable',
                request_serializer=operations__pb2.UserRebaseConfirmableRequest.SerializeToString,
                response_deserializer=operations__pb2.UserRebaseConfirmableResponse.FromString,
                )
        self.UserRevert = channel.unary_unary(
                '/gitaly.OperationService/UserRevert',
                request_serializer=operations__pb2.UserRevertRequest.SerializeToString,
                response_deserializer=operations__pb2.UserRevertResponse.FromString,
                )
        self.UserSquash = channel.unary_unary(
                '/gitaly.OperationService/UserSquash',
                request_serializer=operations__pb2.UserSquashRequest.SerializeToString,
                response_deserializer=operations__pb2.UserSquashResponse.FromString,
                )
        self.UserApplyPatch = channel.stream_unary(
                '/gitaly.OperationService/UserApplyPatch',
                request_serializer=operations__pb2.UserApplyPatchRequest.SerializeToString,
                response_deserializer=operations__pb2.UserApplyPatchResponse.FromString,
                )
        self.UserUpdateSubmodule = channel.unary_unary(
                '/gitaly.OperationService/UserUpdateSubmodule',
                request_serializer=operations__pb2.UserUpdateSubmoduleRequest.SerializeToString,
                response_deserializer=operations__pb2.UserUpdateSubmoduleResponse.FromString,
                )


class OperationServiceServicer(object):
    """OperationService provides an interface for performing mutating git
    operations on a repository on behalf of a user. The user's operation is
    treated as untrusted. Any reference update is thus checked against GitLab's
    '/allowed' endpoint.
    """

    def UserCreateBranch(self, request, context):
        """UserCreateBranch creates a single branch in the context of a specific user. It executes
        hooks and contacts Rails to verify that the user is allowed to create the branch. The
        following known error conditions may happen:

        - `FailedPrecondition` if the Git revision provided in start_point can't be found or updated.
        - `InvalidArgument` if the commit ID resolved from the start_point can't be parsed.
        - `PermissionDenied` with an embedded `UserCreateBranchError` if any custom hooks return a
        non-zero exit code.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UserUpdateBranch(self, request, context):
        """UserUpdateBranch updates a branch to point to a new revision. It executes hooks and
        contacts Rails to verify that the user is allowed to update the branch. The following
        known error conditions may happen:

        - `InvalidArgument` if any of the request fields can't be validated.
        - `Internal` if the newrev or oldrev are invalid in the context of the repository.
        - `UserUpdateBranchResponse` with the `PreReceiveError` field set, if any custom hooks
        return a non-zero exit code.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UserDeleteBranch(self, request, context):
        """UserDeleteBranch force-deletes a single branch in the context of a specific user. It executes
        hooks and contacts Rails to verify that the user is indeed allowed to delete that branch. The
        following known error conditions may happen:

        - Returns `InvalidArgument` in case either the branch name or user are not set.
        - Returns `FailedPrecondition` in case the branch does not exist.
        - Returns `OK` with a `PreReceiveError` in case custom hooks refused the update. If the
        `gitaly_user_delete_branch_structured_errors` feature flag is enabled this error case will
        instead return `PermissionDenied` with either a `CustomHook` or AccessCheck` structured
        error.
        - Returns `FailedPrecondition` in case updating the reference fails because
        of a concurrent write to the same reference. If the
        `gitaly_user_delete_branch_structured_errors` feature flag is set this error case will
        instead return `FailedPrecondition` with a `ReferenceUpdate` structured error.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UserCreateTag(self, request, context):
        """UserCreateTag creates a new tag. This RPC knows to create both lightweight and annotated tags
        depending on whether a message is set.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UserDeleteTag(self, request, context):
        """UserDeleteTag deletes an existing tag. It executes hooks and contacts Rails to verify that
        the user is allowed to delete the tag. The following known error conditions may happen:

        - `InvalidArgument` if the repository, tag_name, user, or expected_old_oid (if provided)
        are invalid.
        - `FailedPrecondition` if the tag_name can't be found, or the ref couldn't be deleted due
        to a concurrent write to the same ref.
        - `Internal` if the tag_name can't be resolved or an unknown error occurs.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UserMergeToRef(self, request, context):
        """UserMergeToRef creates a merge commit and updates target_ref to point to that
        new commit. The first parent of the merge commit (the main line) is taken
        from first_parent_ref. The second parent is specified by its commit ID in source_sha.
        If target_ref already exists it will be overwritten.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UserRebaseToRef(self, request, context):
        """UserRebaseToRef rebases source_sha on top of first_parent_ref and updates
        target_ref to point to that new commit. If target_ref already exists, then
        it will be overwritten. This operation is intended to be used on internal
        refs (e.g. refs/merge-requests/*) and does not call hooks.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UserMergeBranch(self, request_iterator, context):
        """UserMergeBranch tries to merge the given commit into the target branch.
        The merge commit is created with the given user as author/committer and
        the given message.

        This is a two-stage RPC that requires confirmation to make user-visible
        changes to the repository:
        - The first request contains details about the requested merge, which
        will result in a response with the created merge commit ID.
        - The second request should set `apply = true` to apply the merge.

        After the second request, it executes hooks and contacts Rails to verify
        that the user is allowed to update the branch. The following known error
        conditions may happen:

        - `InvalidArgument` if request fields can't be validated or resolved.
        - `NotFound` if the branch can't be found.
        - `FailedPrecondition` if there are merge conflicts, if the merge is
        aborted by setting `apply = false` in the second request, or if the
        merge fails due to a concurrent write to the same ref.
        - `PermissionDenied` if the user doesn't have permissions to merge a branch.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UserFFBranch(self, request, context):
        """UserFFBranch tries to perform a fast-forward merge of a given commit into
        the given branch. If the merge is not a fast-forward merge, the request
        will fail. The RPC will return an empty response in case updating the
        reference fails e.g. because of a race.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UserCherryPick(self, request, context):
        """UserCherryPick tries to perform a cherry-pick of a given commit onto a
        branch.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UserCommitFiles(self, request_iterator, context):
        """UserCommitFiles builds a commit from a stream of actions and updates the target branch to point to it.
        userCommitFilesRequest with a UserCommitFilesRequestHeader must be sent as the first message of the stream.
        Following that, a variable number of actions can be sent to build a new commit. Each action consists of
        a header followed by content if used by the action.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UserRebaseConfirmable(self, request_iterator, context):
        """UserRebaseConfirmable rebases the given remote branch onto a target
        branch. The remote branch may be part of another repository.

        This RPC requires confirmation to make any user-visible changes to the
        repository. The first request sent shall contains details about the
        requested rebase, which will result in a response with the created rebase
        commit ID. Only if a second message with `apply = true` is sent will the
        rebase be applied.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UserRevert(self, request, context):
        """UserRevert tries to perform a revert of a given commit onto a branch.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UserSquash(self, request, context):
        """UserSquash squashes a range of commits into a single commit. If
        successful, it returns the object ID of the newly created squash commit.
        On error, it returns a gRPC error. Some specific errors will have an
        embedded UserSquashError such that clients can deduce what exactly has
        failed.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UserApplyPatch(self, request_iterator, context):
        """UserApplyPatch applies patches to a given branch.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UserUpdateSubmodule(self, request, context):
        """UserUpdateSubmodule updates a submodule to point to a new commit.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_OperationServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'UserCreateBranch': grpc.unary_unary_rpc_method_handler(
                    servicer.UserCreateBranch,
                    request_deserializer=operations__pb2.UserCreateBranchRequest.FromString,
                    response_serializer=operations__pb2.UserCreateBranchResponse.SerializeToString,
            ),
            'UserUpdateBranch': grpc.unary_unary_rpc_method_handler(
                    servicer.UserUpdateBranch,
                    request_deserializer=operations__pb2.UserUpdateBranchRequest.FromString,
                    response_serializer=operations__pb2.UserUpdateBranchResponse.SerializeToString,
            ),
            'UserDeleteBranch': grpc.unary_unary_rpc_method_handler(
                    servicer.UserDeleteBranch,
                    request_deserializer=operations__pb2.UserDeleteBranchRequest.FromString,
                    response_serializer=operations__pb2.UserDeleteBranchResponse.SerializeToString,
            ),
            'UserCreateTag': grpc.unary_unary_rpc_method_handler(
                    servicer.UserCreateTag,
                    request_deserializer=operations__pb2.UserCreateTagRequest.FromString,
                    response_serializer=operations__pb2.UserCreateTagResponse.SerializeToString,
            ),
            'UserDeleteTag': grpc.unary_unary_rpc_method_handler(
                    servicer.UserDeleteTag,
                    request_deserializer=operations__pb2.UserDeleteTagRequest.FromString,
                    response_serializer=operations__pb2.UserDeleteTagResponse.SerializeToString,
            ),
            'UserMergeToRef': grpc.unary_unary_rpc_method_handler(
                    servicer.UserMergeToRef,
                    request_deserializer=operations__pb2.UserMergeToRefRequest.FromString,
                    response_serializer=operations__pb2.UserMergeToRefResponse.SerializeToString,
            ),
            'UserRebaseToRef': grpc.unary_unary_rpc_method_handler(
                    servicer.UserRebaseToRef,
                    request_deserializer=operations__pb2.UserRebaseToRefRequest.FromString,
                    response_serializer=operations__pb2.UserRebaseToRefResponse.SerializeToString,
            ),
            'UserMergeBranch': grpc.stream_stream_rpc_method_handler(
                    servicer.UserMergeBranch,
                    request_deserializer=operations__pb2.UserMergeBranchRequest.FromString,
                    response_serializer=operations__pb2.UserMergeBranchResponse.SerializeToString,
            ),
            'UserFFBranch': grpc.unary_unary_rpc_method_handler(
                    servicer.UserFFBranch,
                    request_deserializer=operations__pb2.UserFFBranchRequest.FromString,
                    response_serializer=operations__pb2.UserFFBranchResponse.SerializeToString,
            ),
            'UserCherryPick': grpc.unary_unary_rpc_method_handler(
                    servicer.UserCherryPick,
                    request_deserializer=operations__pb2.UserCherryPickRequest.FromString,
                    response_serializer=operations__pb2.UserCherryPickResponse.SerializeToString,
            ),
            'UserCommitFiles': grpc.stream_unary_rpc_method_handler(
                    servicer.UserCommitFiles,
                    request_deserializer=operations__pb2.UserCommitFilesRequest.FromString,
                    response_serializer=operations__pb2.UserCommitFilesResponse.SerializeToString,
            ),
            'UserRebaseConfirmable': grpc.stream_stream_rpc_method_handler(
                    servicer.UserRebaseConfirmable,
                    request_deserializer=operations__pb2.UserRebaseConfirmableRequest.FromString,
                    response_serializer=operations__pb2.UserRebaseConfirmableResponse.SerializeToString,
            ),
            'UserRevert': grpc.unary_unary_rpc_method_handler(
                    servicer.UserRevert,
                    request_deserializer=operations__pb2.UserRevertRequest.FromString,
                    response_serializer=operations__pb2.UserRevertResponse.SerializeToString,
            ),
            'UserSquash': grpc.unary_unary_rpc_method_handler(
                    servicer.UserSquash,
                    request_deserializer=operations__pb2.UserSquashRequest.FromString,
                    response_serializer=operations__pb2.UserSquashResponse.SerializeToString,
            ),
            'UserApplyPatch': grpc.stream_unary_rpc_method_handler(
                    servicer.UserApplyPatch,
                    request_deserializer=operations__pb2.UserApplyPatchRequest.FromString,
                    response_serializer=operations__pb2.UserApplyPatchResponse.SerializeToString,
            ),
            'UserUpdateSubmodule': grpc.unary_unary_rpc_method_handler(
                    servicer.UserUpdateSubmodule,
                    request_deserializer=operations__pb2.UserUpdateSubmoduleRequest.FromString,
                    response_serializer=operations__pb2.UserUpdateSubmoduleResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'gitaly.OperationService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class OperationService(object):
    """OperationService provides an interface for performing mutating git
    operations on a repository on behalf of a user. The user's operation is
    treated as untrusted. Any reference update is thus checked against GitLab's
    '/allowed' endpoint.
    """

    @staticmethod
    def UserCreateBranch(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/gitaly.OperationService/UserCreateBranch',
            operations__pb2.UserCreateBranchRequest.SerializeToString,
            operations__pb2.UserCreateBranchResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UserUpdateBranch(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/gitaly.OperationService/UserUpdateBranch',
            operations__pb2.UserUpdateBranchRequest.SerializeToString,
            operations__pb2.UserUpdateBranchResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UserDeleteBranch(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/gitaly.OperationService/UserDeleteBranch',
            operations__pb2.UserDeleteBranchRequest.SerializeToString,
            operations__pb2.UserDeleteBranchResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UserCreateTag(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/gitaly.OperationService/UserCreateTag',
            operations__pb2.UserCreateTagRequest.SerializeToString,
            operations__pb2.UserCreateTagResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UserDeleteTag(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/gitaly.OperationService/UserDeleteTag',
            operations__pb2.UserDeleteTagRequest.SerializeToString,
            operations__pb2.UserDeleteTagResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UserMergeToRef(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/gitaly.OperationService/UserMergeToRef',
            operations__pb2.UserMergeToRefRequest.SerializeToString,
            operations__pb2.UserMergeToRefResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UserRebaseToRef(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/gitaly.OperationService/UserRebaseToRef',
            operations__pb2.UserRebaseToRefRequest.SerializeToString,
            operations__pb2.UserRebaseToRefResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UserMergeBranch(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_stream(request_iterator, target, '/gitaly.OperationService/UserMergeBranch',
            operations__pb2.UserMergeBranchRequest.SerializeToString,
            operations__pb2.UserMergeBranchResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UserFFBranch(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/gitaly.OperationService/UserFFBranch',
            operations__pb2.UserFFBranchRequest.SerializeToString,
            operations__pb2.UserFFBranchResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UserCherryPick(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/gitaly.OperationService/UserCherryPick',
            operations__pb2.UserCherryPickRequest.SerializeToString,
            operations__pb2.UserCherryPickResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UserCommitFiles(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_unary(request_iterator, target, '/gitaly.OperationService/UserCommitFiles',
            operations__pb2.UserCommitFilesRequest.SerializeToString,
            operations__pb2.UserCommitFilesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UserRebaseConfirmable(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_stream(request_iterator, target, '/gitaly.OperationService/UserRebaseConfirmable',
            operations__pb2.UserRebaseConfirmableRequest.SerializeToString,
            operations__pb2.UserRebaseConfirmableResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UserRevert(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/gitaly.OperationService/UserRevert',
            operations__pb2.UserRevertRequest.SerializeToString,
            operations__pb2.UserRevertResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UserSquash(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/gitaly.OperationService/UserSquash',
            operations__pb2.UserSquashRequest.SerializeToString,
            operations__pb2.UserSquashResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UserApplyPatch(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_unary(request_iterator, target, '/gitaly.OperationService/UserApplyPatch',
            operations__pb2.UserApplyPatchRequest.SerializeToString,
            operations__pb2.UserApplyPatchResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UserUpdateSubmodule(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/gitaly.OperationService/UserUpdateSubmodule',
            operations__pb2.UserUpdateSubmoduleRequest.SerializeToString,
            operations__pb2.UserUpdateSubmoduleResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# Copyright 2020 Georges Racinet <georges.racinet@octobus.net>
#
# This software may be used and distributed according to the terms of the
# GNU General Public License version 2 or any later version.
#
# SPDX-License-Identifier: GPL-2.0-or-later
from mercurial import (
    error,
    pathutil,
)


class InvalidPath(ValueError):
    pass


def validate_relative_path(repo, path):
    """Equivalent of `storage.ValidateRelativePath` in Gitaly Golang code.

    Quoting the doc-comment:

      ValidateRelativePath validates a relative path by joining it with rootDir
      and verifying the result is either rootDir or a path within rootDir.
      Returns clean relative path from rootDir to relativePath
      or an ErrRelativePathEscapesRoot if the resulting path is not contained
      within rootDir.

    """
    auditor = pathutil.pathauditor(repo.root, realfs=False)
    try:
        return pathutil.canonpath(repo.root, repo.root, path, auditor)
    except error.Abort:
        raise InvalidPath(path)



# Copyright 2021 Georges Racinet <georges.racinet@octobus.net>
#
# This software may be used and distributed according to the terms of the
# GNU General Public License version 2 or any later version.
#
# SPDX-License-Identifier: GPL-2.0-or-later
import pytest

from mercurial import (
    hg,
)

from ..peer import (
    PeerInitException,
    hg_remote_peer,
)

from ..stub.mercurial_repository_pb2 import (
    MercurialPeer,
)


def raiser(msg):
    raise RuntimeError(msg)


def test_peer_context_init_error(monkeypatch):
    """Errors in peer instantiation are properly re-encapsulated"""
    monkeypatch.setattr(hg, 'peer',
                        lambda *a, **kw: raiser('test-peer-init'))
    with pytest.raises(PeerInitException) as exc_info:
        with hg_remote_peer(
                None,
                MercurialPeer(url="http://heptapod.test/valid/url"),
                storage_path='/does/not/matter/here'):
            pass
    assert exc_info.value.args[0] == 'test-peer-init'


def test_peer_context_caller_error(monkeypatch):
    """Errors while using the peer context are not catched.

    Therefore they can be told apart from those in peer initialization
    """
    monkeypatch.setattr(hg, 'peer', lambda *a, **kw: None)

    with pytest.raises(RuntimeError) as exc_info:
        with hg_remote_peer(
                None,
                MercurialPeer(url="http://heptapod.test/valid/url"),
                storage_path='/does/not/matter/here'):
            raise RuntimeError('error-while-using-peer')

    exc = exc_info.value
    assert not isinstance(exc, PeerInitException)
    assert exc_info.value.args[0] == 'error-while-using-peer'



# Copyright 2023 Georges Racinet <georges.racinet@octobus.net>
#
# This software may be used and distributed according to the terms of the
# GNU General Public License version 2 or any later version.
#
# SPDX-License-Identifier: GPL-2.0-or-later
from heptapod.testhelpers import (
    LocalRepoWrapper,
)

from ..repository import (
    config_inherits,
)


def test_config_inherits(tmpdir):
    wrapper = LocalRepoWrapper.init(tmpdir / 'repo')
    repo = wrapper.repo
    assert config_inherits(repo) is False

    # all other cases are covered by MercurialRepositoryService tests
